"""经验搜索结果模型类型注释文件

此模块中定义了所有现有经验搜索内所有模块的结果模型类型注释类，便于现代编辑器自动补全，提供更好的编码体验。
"""
from datetime import datetime
from typing import List, Union


class JingyanPublisher(object):
    """经验搜索文档发布者模型类型注释类。

    详见`baiduspider.models.jingyan.JingyanPublisher`类。
    """

    def __init__(self) -> None:
        self.name: str
        self.url: str


class JingyanNormal(object):
    """经验搜索普通搜索结果模型类型注释类。

    详见`baiduspider.models.jingyan.JingyanNormal`类。
    """

    def __init__(self) -> None:
        self.title: str
        self.url: str
        self.des: str
        self.pub_date: datetime
        self.category: List[str]
        self.votes: int
        self.publisher: JingyanPublisher
        self.is_original: bool
        self.is_outstanding: bool
        self.plain: dict


class JingyanResult(object):
    """经验搜索结果模型类型注释类。

    详见`baiduspider.models.jingyan.JingyanResult`类。
    """

    def __init__(self) -> None:
        self.results: List[JingyanNormal]
        self.pages: int
        self.total: int



import json
from typing import List

from baiduspider._spider import BaseSpider
from baiduspider.predictor.models import TiebaPredictorResult


class BaiduPredictor(BaseSpider):
    """百度搜索词预测

    本模块爬取百度的搜索词预测部分信息，按照百度提供的默认预测顺序排序，给出百度的搜索词预测
    """

    def __init__(self) -> None:
        super().__init__()
        self.spider_name = "BaiduPredictor"

    def predict_web(self, query: str) -> List[str]:
        """百度网页搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[str]: 百度网页搜索搜索词预测
        """
        data = json.loads(
            self._get_response(
                f"https://www.baidu.com/sugrec?ie=utf-8&json=1&prod=pc&from=pc_web&wd={query}"
            )
        )
        ret = [i["q"] for i in data["g"]]
        if "g" not in data:
            return [data["q"]]
        return ret

    def predict_news(self, query: str) -> List[str]:
        """百度资讯搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[str]: 百度资讯搜索搜索词预测
        """
        data = json.loads(
            self._get_response(f"http://news.baidu.com/sn/api/sug?wd={query}&prod=news")
        )
        return data["data"]

    def predict_pic(self, query: str) -> List[str]:
        """百度图片搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[str]: 百度图片搜索搜索词预测
        """
        data = json.loads(
            self._get_response(
                f"https://www.baidu.com/sugrec?ie=utf-8&wd={query}&prod=open_image"
            )
        )
        ret = [i["q"] for i in data["g"]]
        if "g" not in data:
            return [data["q"]]
        return ret

    def predict_wenku(self, query: str) -> List[str]:
        """百度文库搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[str]: 百度文库搜索搜索词预测
        """
        data = json.loads(
            self._get_response(
                f"https://www.baidu.com/sugrec?prod=open_wenku&wd={query}"
            )
        )
        ret = [i["q"] for i in data["g"]]
        if "g" not in data:
            return [data["q"]]
        return ret

    def predict_zhidao(self, query: str) -> List[str]:
        """百度知道搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[str]: 百度知道搜索搜索词预测
        """
        data = json.loads(
            self._get_response(
                f"https://www.baidu.com/sugrec?wd={query}&prod=open_zhidao"
            )
        )
        if "g" not in data:
            return [data["q"]]
        ret = [i["q"] for i in data["g"]]
        return ret

    def predict_tieba(self, query: str) -> List[TiebaPredictorResult]:
        """百度贴吧搜索搜索词预测

        Args:
            query (str): 要预测的部分搜索词

        Returns:
            List[TiebaPredictorResult]: 百度贴吧搜索搜索词预测
        """
        data = json.loads(
            self._get_response(
                f"https://tieba.baidu.com/suggestion?query={query}&ie=utf-8"
            )
        )
        if data["query_match"]["search_data"] is None:
            return []
        ret = [
            {
                "name": i["fname"],
                "cover": i["fpic"],
                "members": i["member_num"],
                "threads": i["thread_num"],
                "classifiers": [i["fclass1"], i["fclass2"]],
                "desc": i["forum_desc"],
            }
            for i in data["query_match"]["search_data"]
        ]
        return ret



"""知道搜索结果模型类型注释文件

此模块中定义了所有现有知道搜索内所有模块的结果模型类型注释类，便于现代编辑器自动补全，提供更好的编码体验。
"""
from datetime import datetime
from typing import List, Union


class ZhidaoNormal(object):
    """知道搜索普通搜索结果模型类型注释类。

    详见`baiduspider.models.zhidao.ZhidaoNormal`类。
    """

    def __init__(self) -> None:
        self.count: Union[int, None]
        self.date: Union[datetime, None]
        self.answer: Union[str, None]
        self.question: Union[str, None]
        self.agree: int
        self.answerer: str
        self.title: str
        self.url: str
        self.plain: dict


class ZhidaoResult(object):
    """知道搜索结果模型类型注释类。

    详见`baiduspider.models.zhidao.ZhidaoResult`类。
    """

    def __init__(self) -> None:
        self.results: List[ZhidaoNormal]
        self.pages: int
        self.total: int



"""搜索词预测模型类型注释文件

此模块中定义了所有现有搜索词预测模块内所有结果模型类型注释类，便于现代编辑器自动补全，提供更好的编码体验。
"""
from datetime import datetime
from typing import List, Union


class TiebaPredictorResult(object):
    """贴吧搜索搜索结果预测结果模型类型注释类。

    详见`baiduspider.predictor.models.TiebaPredictorResult`类。
    """

    def __init__(self) -> None:
        self.classifiers: List[str]
        self.cover: str
        self.desc: str
        self.members: int
        self.name: str
        self.threads: int



import json
from datetime import time, datetime

import requests
from baiduspider._spider import BaseSpider
from baiduspider.util import handle_err
from bs4 import BeautifulSoup, Comment


class WebSubParser(BaseSpider):
    """网页搜索子解析模块。

    此模块为`BaiduSpider.mobile.search_web`函数的子模块，用于解析网页搜索子模块的HTML代码
    并返回Python字典。
    """

    def __init__(self) -> None:
        super().__init__()
        self.spider_name = "WebSubSpider"

    @handle_err
    def parse_video_block(self, video: BeautifulSoup) -> dict:
        """解析视频子块

        Args:
            video (BeautifulSoup): 从源HTML代码中提取的视频块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if video is None:
            return []
        video = video.find("article")
        tags_container = (
            video.find("section")
            .find("div", class_="c-tags-scroll-wrapper")
            .findAll("li")
        )
        v_url = video["rl-link-href"]
        results = []
        tags = []
        for tag in tags_container:
            text = self._format(tag.text)
            url = tag.find("a")["href"]
            tags.append({"text": text, "url": url})
        vid_container = video.findAll("div", class_="c-span")
        for vid in vid_container:
            vid = vid.find("div", class_="c-touchable-feedback-content")
            url = vid.find("a")["href"]
            _ = vid.find("div", class_="c-img")
            poster = _.find("img")["src"]
            labels = []
            __ = _.find("span", class_="c-label-radius")
            if __ is not None:
                labels.append(self._format(__.text))
            duration = None
            video_num = None
            __ = _.find("div", class_="c-img-info-br")
            if "视频" in __.text:
                video_num = int(self._format(__.text.strip("个视频")))
            elif ":" in __.text:
                duration = __.text
            title = self._format(vid.find("div", role="text").text)
            _ = vid.findAll("div", class_="c-line-clamp1")[-1]
            try:
                author_avatar = _.find("img")["src"]
                __ = _.findAll("span")
                author = self._format(__[0].text)
                if len(__) > 1 and "vid-scroll-pubtime" in __[1]["class"][0]:
                    pub_time = self._format(__[1].text)
                else:
                    pub_time = None
                __ = _.find("span", class_="c-label-radius")
                if __ is not None:
                    labels.append(__.text)
            except:
                author_avatar = None
                author = None
                pub_time = None
            results.append(
                {
                    "poster": poster,
                    "labels": labels,
                    "duration": duration,
                    "video_num": video_num,
                    "title": title,
                    "author_avatar": author_avatar,
                    "author": author,
                    "pub_time": pub_time,
                    "url": url,
                }
            )
        return {"results": results, "tags": tags, "url": v_url}

    @handle_err
    def parse_short_video_block(self, short_video: BeautifulSoup) -> dict:
        """解析短视频子块

        Args:
            short_video (BeautifulSoup): 从源HTML代码中提取的短视频块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if short_video is None:
            return []
        s_total = self._reformat_big_num(
            short_video.find("div", class_="middle")
            .find("span", class_="text")
            .text.strip("条\ue734"),
            "全部",
        )
        s_url = short_video.find("article")["rl-link-href"]
        short_video = short_video.find("section")
        short_video_container = short_video.findAll("div", class_="vid-pocket-item")
        results = []
        for vid in short_video_container:
            poster = vid.find("img")["data-lazy-src"]
            url = vid.parent.parent["href"]
            _ = vid.find("div", class_="c-color-white")
            try:
                author_avatar = _.find("img")["src"]
            except:
                author_avatar = None
            author = _.find("span", class_="c-color-white").text
            play_times = _.findAll("span")[-1].text
            if play_times is not None and "次播放" in play_times:
                delta = 1
                if "万" in play_times:
                    delta = 10000
                elif "亿" in play_times:
                    delta = 100000000
                play_times = int(
                    float(
                        play_times.replace("次播放", "").replace("万", "").replace("亿", "")
                    )
                    * delta
                )
            else:
                play_times = None
            title = vid.find("div", class_="c-gap-bottom-small").text
            results.append(
                {
                    "title": title,
                    "poster": poster,
                    "url": url,
                    "author_avatar": author_avatar,
                    "author": author,
                    "play_times": play_times,
                }
            )
        return {"results": results, "total": s_total, "url": s_url}

    @handle_err
    def parse_baike_block(self, baike: BeautifulSoup) -> dict:
        """解析百科子块

        Args:
            baike (BeautifulSoup): 从源HTML代码中提取的百科块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if baike is None:
            return []
        b_url = json.loads(baike["data-log"])["mu"]
        baike = baike.find("article")
        title = baike.find("h3", class_="c-title").text
        baike = baike.find("section")
        try:
            poster = baike.find("img", class_="c-img-img")["src"]
        except:
            poster = None
        des = baike.find("div", class_="c-abstract").text
        section_container = baike.findAll("a", class_="c-slink")
        sections = []
        for section in section_container:
            text = section.text
            url = section["href"]
            sections.append({"text": text, "url": url})
        _ = baike.find("div", class_="c-source")
        origin = _.find("span", class_="c-color-source").text
        label_container = _.findAll("span", class_="c-label-radius")
        labels = [l.text for l in label_container]
        result = {
            "title": title,
            "poster": poster,
            "des": des,
            "sections": sections,
            "origin": origin,
            "labels": labels,
            "url": b_url,
        }
        return {"result": result}

    @handle_err
    def parse_reyi_block(self, reyi: BeautifulSoup) -> dict:
        """解析热议子块

        Args:
            reyi (BeautifulSoup): 从源HTML代码中提取的热议块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if reyi is None:
            return []
        reyi = reyi.find("section")
        r_url = reyi.find("a", class_="c-blocka")["href"]
        r_total = self._reformat_big_num(
            reyi.find("a", class_="middle").text.strip("条\ue734"), "全部"
        )
        post_container = reyi.findAll("div", class_="tts-b-item")
        posts = []
        for post in post_container:
            _ = post.find("div", role="text")
            author_avatar = _.find("img")["src"]
            __ = _.find("div", class_="c-line-clamp1").findAll("span")
            author = __[0].text
            site = __[1].text
            pub_time = __[-1].text
            _ = post.find("div", role="option")
            des = _.find("p", class_="text-container").text.replace("\ue60b", "")
            __ = _.findAll("img", class_="c-img-img")
            images = []
            for img in __:
                try:
                    images.append(img["data-lazy-src"])
                except:
                    pass
            try:
                origin = _.find("div", class_="origin-content-new").text
            except:
                origin = None
            ___ = post.findAll("i", class_=["c-icon", "c-gap-inner-right-small"])
            __ = []
            for t in ___:
                for c in t["class"]:
                    if "zk-icon-size" in c:
                        __.append(t)
                        break
            __ = __[1:]
            try:
                comments = int(__[0].find_next_sibling("span").text.strip())
            except:
                comments = 0
            try:
                likes = int(__[1].find_next_sibling("span").text.strip())
            except:
                likes = 0
            posts.append(
                {
                    "author_avatar": author_avatar,
                    "author": author,
                    "site": site,
                    "pub_time": pub_time,
                    "des": des,
                    "images": images,
                    "origin": origin,
                    "comments": comments,
                    "likes": likes,
                }
            )
        return {"results": posts, "url": r_url, "total": r_total}

    @handle_err
    def parse_knowledge_block(self, knowledge: BeautifulSoup) -> dict:
        """解析相关知识子块

        Args:
            knowledge (BeautifulSoup): 从源HTML代码中提取的相关知识块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        from pprint import pprint

        if knowledge is None:
            return []
        knowledge = knowledge.find("article")
        k_title = knowledge.find("header").find("h3").text
        knowledge_container = knowledge.findAll("div", class_="c-scroll-item")
        results = []
        for kl in knowledge_container:
            url = kl.find("a")["href"].replace("&amp;", "&")
            image = kl.find("img")["data-lazy-src"]
            title = kl.find("div", class_="c-color-link").text
            try:
                des = kl.find("div", class_="c-color-gray").text
                if not des.strip():
                    des = None
            except AttributeError:
                des = None
            results.append({"url": url, "image": image, "title": title, "des": des})
        return {"results": results, "title": k_title}



import json
from datetime import datetime, time
from html import unescape

from baiduspider._spider import BaseSpider
from baiduspider.errors import ParseError
from baiduspider.mobile.parser.subparser import WebSubParser
from baiduspider.util import handle_err
from bs4 import BeautifulSoup


class MobileParser(BaseSpider):
    def __init__(self) -> None:
        """百度移动端搜索解析器"""
        super().__init__()
        self.webSubParser = WebSubParser()
        self.WEB_NORMAL = "1599"
        self.WEB_VIDEO_NORMAL = "1508"
        self.WEB_VIDEO = "4295"
        self.WEB_SHORT_VIDEO = "4660"
        self.WEB_BAIKE = "1547"
        self.WEB_REYI = "201"
        self.WEB_KNOWLEDGE = "28339"

    @handle_err
    def parse_web(self, content: str) -> dict:
        """解析百度网页搜索的页面源代码.

        Args:
            content (str): 已经转换为UTF-8编码的百度网页搜索HTML源码

        Returns:
            dict: 解析后的结果
        """
        soup = BeautifulSoup(content, "html.parser")
        res_normal_container = soup.findAll(
            "div", class_="c-result", new_srcid=self.WEB_NORMAL
        )
        res_normal = []
        for res in res_normal_container:
            _ = res.find("div", class_="c-result-content").find("article")
            header = _.find("header")
            # 标签
            labels = []
            # 标题
            title = header.find("h3").find("span", class_="c-title-text").text
            # 链接
            try:
                url = _["rl-link-href"]
            except:
                url = header.find("a")["href"]
            __ = header.find("h3").find("span", class_="c-text-public")
            # “官方”标签
            if __ is not None:
                labels.append(__.text)
            section = _.find("section").find("section")
            _ = section.find("div", role="text")
            # （更新）日期
            try:
                date_ = _.find("span", class_="c-color-gray").text
                date = date_.replace("年", "-").replace("月", "-").replace("日", "")
            except:
                date = None
            __ = res.find("section").find("div", class_="c-flexbox")
            ___ = __.find("span", class_="c-text-box")
            # 热度标签
            if ___ is not None:
                labels.append(___.text)
            # 简介
            des = ""
            # 可能有多个`span`标签，需要依次解析
            for s in _.findAll("div"):
                for t in s.findAll("span"):
                    try:
                        if t.find("span").text:
                            continue
                    except:
                        pass
                    try:
                        if "c-color-gray" in t["class"]:
                            continue
                    except:
                        pass
                    des += t.text
                des += "\n"
            des = des.strip("\n")
            # 来源（作者）
            origin = __.find("span", class_="c-footer-showurl")
            if origin is not None:
                origin = origin.text
            else:
                origin = __.find("div", class_="single-text")
                if origin is not None:
                    origin = origin.text
            _ = section.find("a").find("div", role="img")
            # 封面图片链接
            if _ is not None and _.find("img") is not None:
                try:
                    img = _.find("img")["data-lazy-src"].replace("&amp;", "&")
                except:
                    img = _.find("img")["src"].replace("&amp;", "&")
            else:
                img = None
            section_container = res.findAll("a", class_="c-slink")
            sections = []
            for section in section_container:
                text = section.text
                url = section["href"]
                sections.append({"text": text, "url": url})
            res_normal.append(
                {
                    "title": title,
                    "url": url,
                    "labels": labels,
                    "date": date,
                    "des": des,
                    "origin": origin,
                    "img": img,
                    "sections": sections,
                    "type": "result",
                }
            )
        res_video_normal_container = soup.findAll(
            "div", class_="c-result", new_srcid=self.WEB_VIDEO_NORMAL
        )
        res_video_normal = []
        for res in res_video_normal_container:
            title = res.find("span", class_="c-title-text").text
            url = res.find("article")["rl-link-href"].replace("&amp;", "&")
            poster = res.find("img", class_="c-img-img")["src"]
            duration = res.find("div", class_="c-img-info-br").text
            video_num = None
            if "个" in duration:
                video_num = int(duration.strip("个视频"))
                duration = None
            info_container = res.findAll("a", class_="c-blocka")[-1].findAll(
                class_="c-color"
            )
            info = []
            for i in info_container:
                if i.text.startswith("作者："):
                    info.append({"data": i.text.strip("作者：").strip(), "type": "author"})
                elif i.text.startswith("简介："):
                    info.append({"data": i.text.strip("简介：").strip(), "type": "des"})
                elif i.text.startswith("时间："):
                    info.append({"data": i.text.strip("时间：").strip(), "type": "time"})
                elif i.text.startswith("播放："):
                    info.append(
                        {
                            "data": self._reformat_big_num(
                                i.text.strip("播放：").strip("次播放").strip()
                            ),
                            "type": "play_times",
                        }
                    )
                else:
                    info.append({"data": i.text.strip(), "type": "unknown"})
            labels = []
            _ = res.find("div", class_="c-img-radius-tl")
            __ = _.find("span", class_="c-label-radius")
            if __ is not None:
                labels.append(self._format(__.text))
            _ = res.find("div", class_="c-source-new")
            origin = _.find("div", class_="single-text").text
            __ = res.find("section").find("div", class_="c-flexbox")
            ___ = __.find("span", class_="c-text-box")
            # 热度标签
            if ___ is not None:
                labels.append(___.text)
            res_video_normal.append(
                {
                    "title": title,
                    "url": url,
                    "poster": poster,
                    "duration": duration,
                    "info": info,
                    "origin": origin,
                    "labels": labels,
                    "video_num": video_num,
                    "type": "video_normal",
                }
            )
        # 预处理视频子块
        video = soup.find("div", class_="c-result", new_srcid=self.WEB_VIDEO)
        video = self.webSubParser.parse_video_block(video)
        # 预处理短视频子块
        short_video = soup.find(
            "div", class_="c-result", new_srcid=self.WEB_SHORT_VIDEO
        )
        short_video = self.webSubParser.parse_short_video_block(short_video)
        # 预处理百科子块
        baike = soup.find("div", class_="c-result", new_srcid=self.WEB_BAIKE)
        baike = self.webSubParser.parse_baike_block(baike)
        # 预处理相关知识子块
        knowledge = soup.find("div", class_="c-result", new_srcid=self.WEB_KNOWLEDGE)
        knowledge = self.webSubParser.parse_knowledge_block(knowledge)
        pre_results = []
        # 预处理热议子块
        reyi = soup.find("div", class_="c-result", new_srcid=self.WEB_REYI)
        reyi = self.webSubParser.parse_reyi_block(reyi)
        # 加载视频子块
        if video:
            pre_results.append(
                dict(results=video["results"], tags=video["tags"], type="video")
            )
        # 加载短视频子块
        if short_video:
            pre_results.append(
                dict(
                    results=short_video["results"],
                    total=short_video["total"],
                    type="short_video",
                )
            )
        # 加载百科子块
        if baike:
            pre_results.append(dict(result=baike["result"], type="baike"))
        # 加载热议子块
        if reyi:
            pre_results.append(
                dict(
                    results=reyi["results"],
                    url=reyi["url"],
                    total=reyi["total"],
                    type="reyi",
                )
            )
        # 加载相关知识子块
        if knowledge:
            pre_results.append(
                dict(
                    results=knowledge["results"],
                    title=knowledge["title"],
                    type="knowledge",
                )
            )
        results = pre_results
        results.extend(res_normal)
        results.extend(res_video_normal)
        return {"results": results}



"""知道搜索返回值模型模块

此文件定义的所有现有的知道搜索返回值模型并编写了自动构建函数。
"""
from typing import Union

from requests.api import get

from baiduspider.models import convert_time, get_attr
from baiduspider.models.typings.typings_zhidao import *


class ZhidaoNormal(ZhidaoNormal):
    """普通搜索结果模型

    这是一个遵照BaiduSpider知道搜索基本搜索结果结果模型创建的返回模型类。

    Attributes:
        agree (int): 回答赞同数
        answer (str | None): 回答简介
        answerer (str): 回答者
        count (int | None): 回答总数
        date (datetime.datetime): 回答发布日期
        title (str): 问题标题
        question (str | None): 问题简介
        url (str): 问题链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.date = None
        self.answer = None
        self.question = None
        self.answerer = ""
        self.count = None
        self.title = ""
        self.agree = 0
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> ZhidaoNormal:
        __returns = ZhidaoNormal()
        __returns.plain = plain
        __returns.title = get_attr(plain, "title")
        __returns.question = get_attr(plain, "question")
        __returns.answer = get_attr(plain, "answer")
        __returns.count = get_attr(plain, "count")
        __returns.agree = get_attr(plain, "agree")
        __returns.answerer = get_attr(plain, "answerer")
        __returns.date = convert_time(get_attr(plain, "date"))
        __returns.url = get_attr(plain, "url")
        return __returns


class ZhidaoResult(ZhidaoResult):
    """知道搜索结果模型

    这是一个遵照BaiduSpider知道搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[ZhidaoNormal]): 普通搜索结果列表
        pages (int): 搜索结果页数
        total (int): 搜索结果总数
        plain (list): 源搜索结果列表
    """

    def __init__(self) -> None:
        super().__init__()
        self.results = []
        self.pages = 0
        self.total = 0
        self.plain = []

    @staticmethod
    def _build_instance(plain: list, pages: int, total: int) -> ZhidaoResult:
        __returns = ZhidaoResult()
        __returns.plain = plain
        __returns.pages = pages
        __returns.total = total
        for p in plain:
            __returns.results.append(ZhidaoNormal._build_instance(p))
        return __returns

    def __getitem__(self, key) -> Union[ZhidaoNormal, None]:
        return self.results[key]

    def __repr__(self) -> str:
        return "<object ZhidaoResult>"



"""移动端网页搜索结果模型类型注释文件

此模块中定义了所有现有移动端网页搜索内所有模块的结果模型类型注释类，便于现代编辑器自动补全，提供更好的编码体验。
"""
from datetime import datetime, time
from typing import List, Union


class WebVideoDetail(object):
    """网页搜索视频详情搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebVideoDetail`类。
    """

    def __init__(self) -> None:
        self.author: str
        self.author_avatar: str
        self.duration: time
        self.labels: List[str]
        self.poster: str
        self.pub_time: datetime
        self.title: str
        self.url: str
        self.video_num: int
        self.plain: dict


class WebVideoTag(object):
    """网页搜索视频标签搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebVideoTag`类。
    """

    def __init__(self) -> None:
        self.text: str
        self.url: str
        self.plain: dict


class WebVideo(object):
    """网页搜索视频搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebVideo`类。
    """

    def __init__(self) -> None:
        self.results: List[WebVideoDetail]
        self.tags: List[WebVideoTag]
        self.url: str
        self.plain: dict


class WebShortVideoDetail(object):
    """网页搜索短视频详情搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebShortVideoDetail`类。
    """

    def __init__(self) -> None:
        self.author: str
        self.author_avatar: str
        self.play_times: int
        self.poster: str
        self.title: str
        self.url: str
        self.plain: dict


class WebShortVideo(object):
    """网页搜索短视频搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebShortVideo`类。
    """

    def __init__(self) -> None:
        self.results: List[WebShortVideoDetail]
        self.total: int
        self.url: str
        self.plain: dict


class WebSection(object):
    """网页搜索目录搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebSection`类。
    """

    def __init__(self) -> None:
        self.text: str
        self.url: str
        self.plain: dict


class WebBaike(object):
    """网页搜索百科搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebBaike`类。
    """

    def __init__(self) -> None:
        self.des: str
        self.labels: List[str]
        self.origin: str
        self.poster: str
        self.sections: List[WebSection]
        self.title: str
        self.url: str
        self.plain: dict


class WebReyiDetail(object):
    """网页搜索热议详情搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebReyiDetail`类。
    """

    def __init__(self) -> None:
        self.author: str
        self.author_avatar: str
        self.comments: int
        self.des: str
        self.images: List[str]
        self.likes: int
        self.origin: str
        self.pub_time: datetime
        self.site: str
        self.plain: dict


class WebReyi(object):
    """网页搜索热议搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebReyi`类。
    """

    def __init__(self) -> None:
        self.results: List[WebReyiDetail]
        self.total: int
        self.url: str
        self.plain: dict


class WebKnowledgeDetail(object):
    """网页搜索相关知识详情搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebKnowledgeDetail`类。
    """

    def __init__(self) -> None:
        self.des: str
        self.image: str
        self.title: str
        self.url: str
        self.plain: dict


class WebKnowledge(object):
    """网页搜索相关知识搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebKnowledge`类。
    """

    def __init__(self) -> None:
        self.results: List[WebKnowledgeDetail]
        self.title: str
        self.plain: dict


class WebNormal(object):
    """网页搜索普通搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebNormal`类。
    """

    def __init__(self) -> None:
        self.des: str
        self.image: str
        self.title: str
        self.url: str
        self.sections: List[WebSection]
        self.plain: dict


class WebVideoNormalInfo(object):
    """网页搜索普通视频信息搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebVideoNormaInfo`类。
    """

    def __init__(self) -> None:
        self.data: str
        self.type: str
        self.plain: dict


class WebVideoNormal(object):
    """网页搜索普通视频搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.WebVideoNormal`类。
    """

    def __init__(self) -> None:
        self.title: str
        self.url: str
        self.poster: str
        self.duration: time
        self.info: List[WebVideoNormalInfo]
        self.origin: str
        self.labels: List[str]
        self.video_num: int
        self.plain: dict


class WebResult(object):
    """网页搜索搜索结果模型类型注释类。

    详见`baiduspider.mobile.models.web.Web`类。
    """

    def __init__(self) -> None:
        self.video: Union[WebVideo, None]
        self.short_video: Union[WebShortVideo, None]
        self.baike: Union[WebBaike, None]
        self.reyi: Union[WebReyi, None]
        self.normal: List[WebNormal]
        self.knowledge: Union[WebKnowledge, None]
        self.video_normal: List[WebVideoNormal]
        self.query: str
        self.plain: dict



"""BaiduSpider，一个爬取百度的利器.

:Author: Sam Zhang

:Licence: GPL_V3

:GitHub: https://github.com/BaiduSpider/BaiduSpider
"""
import datetime
import time as time_lib
from typing import Tuple, Union
from urllib.parse import quote

from baiduspider._spider import BaseSpider
from baiduspider.parser import Parser
from baiduspider.models.web import WebResult
from baiduspider.models.pic import PicResult
from baiduspider.models.news import NewsResult
from baiduspider.models.video import VideoResult
from baiduspider.models.zhidao import ZhidaoResult
from baiduspider.models.wenku import WenkuResult
from baiduspider.models.jingyan import JingyanResult
from baiduspider.models.baike import BaikeResult
import hashlib
import random

__all__ = ["BaiduSpider"]


class BaiduSpider(BaseSpider):
    """爬取百度的搜索结果.

    本类的所有成员方法都遵循下列格式：

        {
            'results': <一个列表，表示搜索结果，内部的字典会因为不同的成员方法而改变>,
            'total': <一个正整数，表示搜索结果的最大页数，可能会因为搜索结果页码的变化而变化，因为百度不提供总共的搜索结果页数>
        }

    目前支持百度搜索，百度图片，百度知道，百度视频，百度资讯，百度文库，百度经验和百度百科，并且返回的搜索结果无广告。继承自``BaseSpider``。

    - BaiduSpider.`#!python search_web(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        exclude: list = [],
        time: Union[tuple, str, None] = None,
        proxies: dict = None
    ) -> WebResult`: 百度网页搜索

    - BaiduSpider.`#!python search_pic(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        proxies: dict = None
    ) -> PicResult`: 百度图片搜索

    - BaiduSpider.`#!python search_zhidao(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        time: Union[str, None] = None,
        proxies: dict = None
    ) -> ZhidaoResult`: 百度知道搜索

    - BaiduSpider.`#!python search_video(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        proxies: dict = None
    ) -> VideoResult`: 百度视频搜索

    - BaiduSpider.`#!python search_news(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        sort_by: str = "focus",
        show: str = "all",
        proxies: dict = None
    ) -> NewsResult`: 百度资讯搜索

    - BaiduSpider.`#!python search_wenku(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        scope: str = "all",
        format: str = "all",
        time: str = "all",
        page_range: Union[Tuple[int], str] = "all",
        sort_by: str = "relation",
        proxies: dict = None
    ) -> WenkuResult`: 百度文库搜索

    - BaiduSpider.`#!python search_jingyan(
        self: BaiduSpider,
        query: str,
        pn: int = 1,
        scope: str = "all",
        proxies: dict = None
    ) -> JingyanResult`: 百度经验搜索

    - BaiduSpider.`#!python search_baike(
        self: BaiduSpider,
        query: str,
        proxies: dict = None
    )`: 百度百科搜索
    """

    def __init__(self, cookie: str = None) -> None:
        """初始化BaiduSpider.

        - 设置Cookie：
            ```python
            spider = BaiduSpider(cookie="你的cookie")
            ```
            Cookie可以被用于增强爬虫的真实性，尽可能使百度减少封禁IP的最大限制。

            如果你想获取你的Cookie，请打开<https://www.baidu.com/s?wd=placeholder&pn=0>，并
            按F12打开开发者工具，然后在开发者工具最上方的选项栏中选择“网络”（Network）这一选项，点击
            出现的列表中最上方的以`s?wd=placeholder`开头的选项，在出现的详情中找到`Request Headers`
            一项，然后在它的下方找到`Cookie`，并复制Cookie这一选项内（不包括`Cookie: `）后面的所有内容，
            并将它粘贴在你需要的位置。

            请勿传入非法的Cookie。

        Args:
            cookie (Union[str, None], optional): 浏览器抓包得到的cookie. Defaults to None.
        """
        super().__init__()
        # 爬虫名称（不是请求的，只是用来表识）
        self.spider_name = "BaiduSpider"
        # 解析Cookie
        if cookie is not None:
            if cookie.find("__yjs_duid") == -1:
                cookie += "; __yjs_duid=1_" + str(hashlib.md5().hexdigest()) + "; "
            else:
                _ = cookie.split("__yjs_duid=")
                __ = _[1].split(";", 1)[-1]
                ___ = hashlib.md5()
                cookie = _[0] + "__yjs_duid=1_" + str(___.hexdigest()) + "; " + __
        # 设置请求头
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; WOW64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/86.0.4240.111 Safari/537.36",
            "Referer": "https://www.baidu.com/s?ie=utf-8&f=8&rsv_bp=1&rsv_idx=2&ch=&tn=baiduhome_pg&bar=&wd=123&oq=123&rsv_pq=896f886f000184f4&rsv_t=fdd2CqgBgjaepxfhicpCfrqeWVSXu9DOQY5WyyWqQYmsKOC%2Fl286S248elzxl%2BJhOKe2&rqlang=cn",
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
            "Sec-Fetch-Mode": "navigate",
            "Cookie": cookie,
        }
        self.parser = Parser()
        self.EMPTY = {"results": [], "pages": 0}
        self.RESULTS_PER_PAGE = {
            "web": 10,
            "pic": 20,
            "zhidao": 10,
            "news": 10,
            "jingyan": 10,
        }

    def search_web(
        self,
        query: str,
        pn: int = 1,
        exclude: list = [],
        time: Union[tuple, str, None] = None,
        proxies: dict = None,
    ) -> WebResult:
        """百度网页搜索。

        - 简单搜索：
            ```python
            BaiduSpider().search_web('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'result': int,  # 总计搜索结果数,
                        'type': 'total'  # type用来区分不同类别的搜索结果
                    },
                    {
                        'results': [
                            str,  # 相关搜索建议
                            '...',
                            '...',
                            '...',
                            ...
                        ],
                        'type': 'related'
                    },
                    {
                        'process': str,  # 算数过程
                        'result': str,  # 运算结果
                        'type': 'calc'
                        # 这类搜索结果仅会在搜索词涉及运算时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'results': [
                            {
                                'author': str,  # 新闻来源
                                'time': str,  # 新闻发布时间
                                'title': str,  # 新闻标题
                                'url': str,  # 新闻链接
                                'des': str,  # 新闻简介，大部分情况为None
                            },
                            { ... },
                            { ... },
                            { ... },
                            ...
                        ],
                        'type': 'news'
                        # 这类搜索结果仅会在搜索词有相关新闻时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'results': [
                            {
                                'cover': str,  # 视频封面图片链接
                                'origin': str,  # 视频来源
                                'length': str,  # 视频时长
                                'title': str,  # 视频标题
                                'url': str,  # 视频链接
                            },
                            { ... },
                            { ... },
                            { ... },
                            ...
                        ],
                        'type': 'video'
                        # 这类搜索结果仅会在搜索词有相关视频时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'result': {
                            'cover': str,  # 百科封面图片/视频链接
                            'cover-type': str,  # 百科封面类别，图片是image，视频是video
                            'des': str,  # 百科简介
                            'title': str,  # 百科标题
                            'url': str,  # 百科链接
                        },
                        'type': 'baike'
                        # 这类搜索结果仅会在搜索词有相关百科时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'result': {
                            'cover': str,  # 贴吧封面图片链接
                            'des': str,  # 贴吧简介
                            'title': str,  # 贴吧标题
                            'url': str,  # 贴吧链接
                            'followers': str,  # 贴吧关注人数（可能有汉字，如：1万）
                            'hot': [{  # list, 热门帖子
                                'clicks': str,  # 帖子点击总数
                                'replies': str,  # 帖子回复总数
                                'title': str,  # 帖子标题
                                'url': str,  # 帖子链接
                            }],
                            'total': str,  # 贴吧总帖子数（可能有汉字，如：17万）'
                        },
                        'type': 'tieba'
                        # 这类搜索结果仅会在搜索词有相关贴吧时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'result': {
                            'blogs': [{  # list, 博客列表
                                'des': str,  # 博客简介，没有时为`None`
                                'origin': str,  # 博客来源
                                'tags': [  # list, 博客标签
                                    str,  # 标签文字
                                ],
                                'title': str,  # 博客标题
                                'url': str,  # 博客链接
                            }],
                            'title': str,  # 博客搜索标题
                            'url': str,  # 博客搜索链接 (https://kaifa.baidu.com)
                        },
                        'type': 'blog'
                        # 这类搜索结果仅会在搜索词有相关博客时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'result': {
                            'title': str,  # 仓库标题
                            'des': str,  # 仓库简介
                            'url': str,  # 仓库链接
                            'star': int,  # 仓库star数
                            'fork': int,  # 仓库fork数
                            'watch': int,  # 仓库watch数
                            'license': str,  # 仓库版权协议
                            'lang': str,  # 仓库使用的编程语言
                            'status': str,  # 仓库状态图表链接
                        },
                        'type': 'gitee'
                        # 这类搜索结果仅会在搜索词有相关代码仓库时出现，不一定每个搜索结果都会出现的
                    },
                    {
                        'result': {
                            {
                                'songs': [{  # list, 歌曲信息
                                    'album': {  # dict, 歌曲专辑
                                        'name': str,  # 专辑名称
                                        'url': str,  # 专辑链接
                                    },
                                    'singer': [{  # list, 歌手信息
                                        'name': str,  # 歌手名称
                                        'url': str,  # 歌手链接
                                    }],
                                    'song': {  # dict, 歌曲信息
                                        'copyright': bool,  # 歌曲是否有版权
                                        'duration': datetime.time,  # 歌曲时长
                                        'is_original': bool,  # 歌曲是否为原唱
                                        'labels': List[str],  # 歌曲标签
                                        'name': str,  # 歌曲名称
                                        'other_sites: List[str],  # 歌曲在其他网站发布的链接
                                        'poster': str,  # 歌曲海报图片链接
                                        'pub_company': str,  # 歌曲发行公司名称
                                        'pub_date': datetome.datetime,  # 歌曲发行时间
                                        'site': str,  # 歌曲发布网站名称（拼音）
                                        'url': str  # 歌曲链接
                                    }
                                }],
                                'title': str,  # 音乐标题
                                'url': str  # 音乐链接
                            }
                        },
                        'type': 'music'
                        # 这类搜索结果仅会在搜索词有相关音乐时出现，不一定每个搜索结果都会出现的
                    }
                    {
                        'des': str,  # 搜索结果简介
                        'origin': str,  # 搜索结果的来源，可能是域名，也可能是名称
                        'time': str,  # 搜索结果的发布时间
                        'title': str,  # 搜索结果标题
                        'type': 'result,  # 正经的搜索结果
                        'url': str  # 搜索结果链接
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 总计的搜索结果页数，可能会因为当前页数的变化而随之变化
            }
            ```

        - 带页码：
            ```python
            BaiduSpider().search_web('搜索词', pn=2)
            ```

        - 按需解析：
            ```python
            BaiduSpider().search_web('搜索词', exclude=['要屏蔽的子部件列表'])
            ```
            可选值：`['news', 'video', 'baike', 'tieba', 'blog', 'gitee', 'related', 'calc', 'music']`，
            分别表示：资讯，视频，百科，贴吧，博客，Gitee代码仓库，相关搜索，计算。
            当`exclude=['all']`时，将仅保留基本搜索结果和搜索结果总数。
            如果`all`在`exclude`列表里，则将忽略列表中的剩余部件，返回`exclude=['all']`时的结果。

        - 按时间筛选：
            ```python
            BaiduSpider().search_web('搜索词', time=(开始时间, 结束时间))
            ```
            其中，开始时间和结束时间均为datetime.datetime类型，或者是使用time.time()函数生成的时间戳。
            time参数也可以是以下任意一个字符串：`['day', 'week', 'month', 'year']`。它们分别表示：一天内、
            一周内、一月内、一年内。当`time`为`None`时，BaiduSpider将展示全部结果，忽略筛选。
            如果参数非法，BaiduSpider会忽略此次筛选。

        - 设置代理：
            ```python
            BaiduSpider().search_web('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要爬取的搜索词.
            pn (int, optional): 爬取的页码. Defaults to 1.
            exclude (list, optional): 要屏蔽的控件. Defaults to [].
            time (Union[tuple, str, None]): 按时间筛选参数. Defaults to None.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            WebResult: 爬取的返回值和搜索结果
        """
        error = None
        result = self.EMPTY
        # 按需解析
        if "all" in exclude:
            exclude = [
                "news",
                "video",
                "baike",
                "tieba",
                "blog",
                "gitee",
                "calc",
                "related",
                "music",
            ]
        # 按时间筛选
        if type(time) == str:
            to = datetime.datetime.now()
            from_ = datetime.datetime(
                to.year, to.month, to.day, to.hour, to.minute, to.second, to.microsecond
            )
            if time == "day":
                from_ += datetime.timedelta(days=-1)
            elif time == "week":
                from_ += datetime.timedelta(days=-7)
            elif time == "month":
                from_ += datetime.timedelta(days=-31)
            elif time == "year":
                from_ += datetime.timedelta(days=-365)
        elif type(time) == tuple or type(time) == list:
            from_ = time[0]
            to = time[1]
        else:
            to = from_ = None
        if type(to) == datetime.datetime and type(from_) == datetime.datetime:
            FORMAT = "%Y-%m-%d %H:%M:%S"
            to = int(time_lib.mktime(time_lib.strptime(to.strftime(FORMAT), FORMAT)))
            from_ = int(
                time_lib.mktime(time_lib.strptime(from_.strftime(FORMAT), FORMAT))
            )
        try:
            text = quote(query, "utf-8")
            url = "https://www.baidu.com/s?wd=%s&pn=%d&inputT=%d" % (
                text,
                (pn - 1) * 10,
                random.randint(500, 4000),
            )
            if to is not None and from_ is not None:
                url += "&gpc=" + quote(f"stf={from_},{to}|stftype=2")
            # 解析Cookie
            cookie = self.headers["Cookie"]
            if cookie is not None:
                if cookie.find("__yjs_duid") == -1:
                    pass
                else:
                    _ = cookie.split("__yjs_duid=")
                    __ = _[1].split(";", 1)[-1]
                    ___ = hashlib.md5()
                    cookie = _[0] + "__yjs_duid=1_" + str(___.hexdigest()) + __
            self.headers["Cookie"] = cookie
            content = self._get_response(url, proxies)
            # print(content)
            results = self.parser.parse_web(content, exclude=exclude)
        except Exception as err:
            error = err
        finally:
            self._handle_error(error, "BaiduSpider", "parse-web")
        pages = self._calc_pages(results["total"], self.RESULTS_PER_PAGE["web"])
        return WebResult._build_instance(
            plain=results["results"], pages=pages, total=results["total"]
        )

    def search_pic(self, query: str, pn: int = 1, proxies: dict = None) -> PicResult:
        """百度图片搜索。

        - 实例：
            ```python
            BaiduSpider().search_pic('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'host': str,  # 图片来源域名
                        'title': str,  # 图片标题
                        'url': str,  # 图片链接
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 搜索结果总计页码，可能会变化
            }
            ```

        - 带页码的搜索：
            ```python
            BaiduSpider().search_pic('搜索词', pn=2)
            ```

        - 设置代理：
            ```python
            BaiduSpider().search_pic('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要爬取的query
            pn (int, optional): 爬取的页码. Defaults to 1.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            PicResult: 爬取的搜索结果
        """
        error = None
        result = self.EMPTY
        try:
            url = "http://image.baidu.com/search/flip?tn=baiduimage&word=%s&pn=%d" % (
                quote(query),
                (pn - 1) * 20,
            )
            content = self._get_response(url, proxies)
            result = self.parser.parse_pic(content)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            self._handle_error(error)
        pages = self._calc_pages(result["total"], self.RESULTS_PER_PAGE["pic"])
        return PicResult._build_instance(
            plain=result["results"], pages=pages, total=result["total"]
        )

    def search_zhidao(
        self,
        query: str,
        pn: int = 1,
        time: Union[str, None] = None,
        proxies: dict = None,
    ) -> ZhidaoResult:
        """百度知道搜索。

        - 普通搜索：
            ```python
            BaiduSpider().search_zhidao('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'count': int,  # 回答总数
                        'date': str,  # 回答发布日期
                        'question': str,  # 问题简介
                        'answer': str,  # 回答简介
                        'agree': int,  # 回答赞同数
                        'answerer': str,  # 回答者
                        'title': str,  # 问题标题
                        'url': str  # 问题链接
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 搜索结果最大页数，可能会变化
            }
            ```

        - 带页码的搜索：
            ```python
            BaiduSpider().search_zhidao('搜索词', pn=2)
            ```

        - 按时间筛选：
            ```python
            BaiduSpider().search_zhidao('搜索词', time='时间范围')
            ```
            其中，time参数可以是以下任意一个字符串：['week', 'month', 'year']。它们分别表示：一周内、一月内、
            一年内。当`time`为`None`时，BaiduSpider将展示全部结果，忽略筛选。
            如果参数非法，BaiduSpider会忽略此次筛选。

        - 设置代理：
            ```python
            BaiduSpider().search_zhidao('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要搜索的query
            pn (int, optional): 搜索结果的页码. Defaults to 1.
            time (Union[str, None], optional): 时间筛选参数. Defaults to None.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            dict: 搜索结果以及总页码
        """
        error = None
        result = self.EMPTY
        _ = [None, None, "week", "month", "year"]
        if _.index(time) > 0:
            time = _.index(time)
        else:
            time = 0
        try:
            url = (
                "https://zhidao.baidu.com/search?lm=0&rn=10&fr=search&pn=%d&word=%s&date=%d"
                % ((pn - 1) * 10, quote(query), time)
            )
            code = self._get_response(url, proxies, "gb2312")
            # 转化编码
            # source.encoding = "gb2312"
            # code = source.text
            result = self.parser.parse_zhidao(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        pages = self._calc_pages(result["total"], self.RESULTS_PER_PAGE["zhidao"])
        return ZhidaoResult._build_instance(result["results"], pages, result["total"])

    def search_video(
        self, query: str, pn: int = 1, proxies: dict = None
    ) -> VideoResult:
        """百度视频搜索。

        - 普通搜索：
            ```python
            BaiduSpider().search_video('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'img': str,  # 视频封面图片链接
                        'time': str,  # 视频时长
                        'title': str,  # 视频标题
                        'url': str  # 视频链接
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                'pages': int  # 搜索结果最大页数，可能因搜索页数改变而改变
            }
            ```

        - 带页码：
            ```python
            BaiduSpider().search_video('搜索词', pn=2)
            ```

        - 设置代理：
            ```python
            BaiduSpider().search_video('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要搜索的query
            pn (int, optional): 搜索结果的页码. Defaults to 1.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            VideoResult: 爬取的搜索结果
        """
        error = None
        result = self.EMPTY
        try:
            url = (
                "https://www.baidu.com/sf/vsearch?pd=video&tn=vsearch&wd=%s&pn=%d&async=1"
                % (quote(query), (pn - 1) * 10)
            )
            # 获取源码
            code = self._get_response(url, proxies)
            result = self.parser.parse_video(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        return VideoResult._build_instance(result["results"])

    def search_news(
        self,
        query: str,
        pn: int = 1,
        sort_by: str = "focus",
        show: str = "all",
        proxies: dict = None,
    ) -> NewsResult:
        """百度资讯搜索。

        - 获取资讯搜索结果：
            ```python
            BaiduSpider().search_news('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'author': str,  # 资讯来源（作者）
                        'date': str,  # 资讯发布时间
                        'des': str,  # 资讯简介
                        'title': str,  # 资讯标题
                        'url': str  # 资讯链接
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 搜索结果最大页码，可能会因为当前页数变化而变化
            }
            ```

        - 带页码：
            ```python
            BaiduSpider().search_news('搜索词', pn=2)
            ```

        - 排序方式：
            目前支持两种排序方式：按焦点排序（默认）和按时间排序。可以通过设置`sort_by`参数来设置排序方式。`sort_by`
            参数只支持两种值：`focus`（按焦点排序，默认值）和`time`（按时间排序）。样例：
            ```python
            BaiduSpider().search_news('搜索词', sort_by='time')  # 按时间排序
            ```

        - 筛选：
            你可以通过设置`show`参数设置要筛选显示的资讯来源。目前支持三种来源：`all`（全部显示，默认）、`media`（媒体）
            和`baijiahao`（百家号）。样例：
            ```python
            BaiduSpider().search_news('搜索词', show='media')  # 仅显示来自媒体的新闻结果
            ```

        - 设置代理：
            ```python
            BaiduSpider().search_news('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 搜索query
            pn (int, optional): 搜索结果的页码. Defaults to 1.
            sort_by (str, optional): 搜索结果排序方式. Defaults to "focus".
            show (str, optional): 搜索结果筛选方式. Defaults to "all".
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            NewsResult: 爬取的搜索结果与总页码。
        """
        error = None
        result = self.EMPTY
        try:
            if sort_by == "time":
                sort_by = 4
            else:
                sort_by = 1
            if show == "media":
                show = 1
            elif show == "baijiahao":
                show = 2
            else:
                show = 0
            url = (
                "https://www.baidu.com/s?tn=news&wd=%s&pn=%d&rtt=%d&medium=%d&cl=2"
                % (quote(query), (pn - 1) * 10, sort_by, show)
            )
            # 源码
            code = self._get_response(url, proxies)
            result = self.parser.parse_news(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        pages = self._calc_pages(result["total"], self.RESULTS_PER_PAGE["news"])
        return NewsResult._build_instance(result["results"], pages, result["total"])

    def search_wenku(
        self,
        query: str,
        pn: int = 1,
        scope: str = "all",
        format: str = "all",
        time: str = "all",
        page_range: Union[Tuple[int], str] = "all",
        sort_by: str = "relation",
        proxies: dict = None,
    ) -> WenkuResult:
        """百度文库搜索。

        请注意，目前百度文库搜索若报错，则可能需要先手动打开百度文库搜索
        （`https://wenku.baidu.com/search?word=placeholder&lm=0&od=0&fr=top_home&ie=utf-8`）
        通过安全验证后才能正常搜索。

        - 普通搜索：
            ```python
            BaiduSpider().search_wenku('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'pub_date': str,  # 文档发布日期
                        'des': str,  # 文档简介
                        'downloads': int,  # 文档下载量
                        'pages': int,  # 文档页数
                        'title': str,  # 文档标题
                        'type': str,  # 文档格式，为全部大写字母
                        'url': str,  # 文档链接
                        'quality': float,  # 文档质量分
                        'uploader': {  # dict, 文档上传者信息
                            'name': str,  # 文档上传者用户名
                            'url': str  # 文档上传者链接
                        },
                        'is_vip': bool  # 该文档是否需要VIP权限下载
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 总计搜索结果的页数
            }
            ```

        - 带页码的搜索：
            ```python
            BaiduSpider().search_wenku('搜索词', pn=2)
            ```

        - 筛选范围：
            BaiduSpider支持五种筛选范围：全部、VIP专享、VIP免费、免费、精品。你可以使用`scope`参数来定义此次搜索
            范围。`scope`参数的取值可以是下列任一一项：['all', 'vip-only', 'vip-free', 'free', 'high-quality']。
            他们的含义与上文所述一致。样例：
            ```python
            BaiduSpider().search_wenku('搜索词', scope='free')  # 仅显示免费的文档
            ```
            若`scope`参数非法，BaiduSpider将忽略此次筛选。

        - 格式筛选：
            BaiduSpider支持六种格式筛选。你可以通过设置`format`参数来设置需要筛选的格式。`format`参数取值可以是：
            `all`（全部）、`doc`（DOC文档）、`ppt`（幻灯片文档）、`txt`（纯文本文档）、`pdf`（PDF文档）、`xls`
            （Excel表格文档）。例如：
            ```python
            BaiduSpider().search_wenku('搜索词', format='ppt')  # 仅显示幻灯片文档
            ```
            若`format`参数非法，BaiduSpider将忽略此次筛选。

        - 按时间筛选：
            BaiduSpider提供百度文科搜索的按时间筛选。`time`参数接受的合法传参如下：['all', 'this-year', 'last-year',
            'previous-years']。他们分别表示全部、今年、去年和前年及以前。示例：
            ```python
            BaiduSpider().search_wenku('搜索词', time='last-year')  # 仅显示去年上传的文档
            ```
            若`time`参数非法，BaiduSpider将忽略此次筛选。

        - 按页数筛选：
            BaiduSpider提供使用页数筛选文档，参数为`page_range`。`page_range`可选值为：['all', Tuple[start: int, end: int]]。
            分别表示全部和Tuple[开始页码(`int`), 结束页码(`int`)]。样例：
            ```python
            BaiduSpider().search_wenku('搜索词', page_range=(0, 10))  # 仅显示页数为0 - 10页的文档
            ```
            若`page_range`参数非法，BaiduSpider将忽略此次筛选。

        - 搜索结果排序：
            BaiduSpider提供由百度文库内置的搜索结果排序。你可以通过设置`sort_by`参数来设置排序方式，默认为按相关性排序。`sort_by`可选值为：
            ['relation', 'time', 'downloads', 'score']，分别表示按相关性、按时间、按下载量和按评分排序。样例：
            ```python
            BaiduSpider().search_wenku('搜索词', sort_by='downloads')  # 按下载量排序
            ```
            若`sort_by`参数非法，BaiduSpider将忽略此次筛选。

        - 设置代理：
            ```python
            BaiduSpider().search_wenku('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要搜索的query
            pn (int, optional): 搜索的页码. Defaults to 1.
            scope (str, optional): 按范围筛选. Defaults to "all".
            format (str, optional): 按格式筛选. Defaults to "all".
            time (str, optional): 按时间筛选. Defaults to "all".
            page_range (Union[str, Tuple[int]]): 按页数筛选. Defaults to "all".
            sort_by (str): 排序方式. Defaults to "relation".
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            WenkuResult: 搜索结果和总计页数
        """
        error = None
        result = self.EMPTY
        # 范围筛选
        _ = ["all", "vip-only", "vip", "free", "high-quality"]
        if _.index(scope) > 0:
            scope = _.index(scope)
        else:
            scope = 0
        # 格式筛选
        _ = ["all", "doc", "pdf", "ppt", "xls", "txt"]
        if _.index(format) > 0:
            format = _.index(format)
        else:
            format = 0
        # 按时间筛选
        _ = ["all", "this-year", "last-year", "previous-years"]
        if _.index(time) > 0:
            time = _.index(time)
        else:
            time = 0
        # 按页数筛选
        if (
            type(page_range) is tuple
            and len(page_range) == 2
            and type(page_range[0]) is int
            and type(page_range[1]) is int
        ):
            pass
        else:
            page_range = -1
        # 排序方式
        _ = ["relation", "time", "downloads", "score"]
        if _.index(sort_by) > 0:
            sort_by = _.index(sort_by)
        else:
            sort_by = 0
        try:
            url = (
                "https://wenku.baidu.com/gsearch/search/pcsearch?word=%s&pn=%d&fr=top_home&fd=%d&lm=%d&pt=%d&od=%d"
                % (quote(query), (pn - 1) * 10, scope, format, time, sort_by)
            )
            if page_range != -1:
                url += "&pb=%d&pe=%d" % (page_range[0], page_range[1])
            else:
                url += "&pg=0"
            code = self._get_response(url, proxies)
            result = self.parser.parse_wenku(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        return WenkuResult._build_instance(result["results"], result["pages"])

    def search_jingyan(
        self, query: str, pn: int = 1, scope: str = "all", proxies: dict = None
    ) -> JingyanResult:
        """百度经验搜索。

        - 例如：
            ```python
            BaiduSpider().search_jingyan('关键词')
            ```
            `plain`返回值：
            ```python
            {
                'results': [
                    {
                        'title': str,  # 经验标题
                        'url': str,  # 经验链接
                        'des': str,  # 经验简介
                        'date': str,  # 经验发布日期
                        'category': List[str],  # 经验分类
                        'votes': int,  # 经验的支持票数
                        'publisher': {  # dict, 经验发布者信息
                            'name': str,  # 经验发布者用户名
                            'url': str  # 经验发布者链接
                        },
                        'is_original': bool,  # 经验是否为原创
                        'is_outstanding': bool  # 经验是否为优秀经验
                    },
                    { ... },
                    { ... },
                    { ... },
                    ...
                ],
                'pages': int  # 总计搜索结果页数
            }
            ```

        - 带页码的：
            ```python
            BaiduSpider().search_jingyan('搜索词', pn=2)
            ```

        - 筛选经验：
            你可以通过设置`scope`参数来设定筛选范围。`scope`参数的可选值为：["all", "outstanding", "praise", "original"]。
            它们分别表示：全部经验、优秀经验、最受好评和原创经验。例子：
            ```python
            BaiduSpider().search_jingyan('搜索词', scope="outstanding")  # 仅显示优秀经验
            ```

        - 设置代理：
            ```python
            BaiduSpider().search_jingyan('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要搜索的关键词
            pn (int, optional): 搜索结果的页码. Defaults to 1.
            scope (str, optional): 筛选范围. Defaults to "all".
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            JingyanResult: 搜索结果以及总计的页码.
        """
        error = None
        result = self.EMPTY
        _ = ["all", "outstanding", "praise", "original"]
        if _.index(scope) > 0:
            scope = _.index(scope)
        else:
            scope = 0
        try:
            url = "https://jingyan.baidu.com/search?word=%s&pn=%d&lm=%d" % (
                quote(query),
                (pn - 1) * 10,
                scope,
            )
            code = self._get_response(url, proxies)
            result = self.parser.parse_jingyan(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        pages = self._calc_pages(result["total"], self.RESULTS_PER_PAGE["jingyan"])
        return JingyanResult._build_instance(result["results"], pages, result["total"])

    def search_baike(self, query: str, proxies: dict = None) -> BaikeResult:
        """百度百科搜索。

        - 使用方法：
            ```python
            BaiduSpider().search_baike('搜索词')
            ```
            `plain`返回值：
            ```python
            {
                'results': {
                    [
                        'title': str,  # 百科标题
                        'des': str,  # 百科简介
                        'date': str,  # 百科最后更新时间
                        'url': str  # 百科链接
                    ],
                    [ ... ],
                    [ ... ],
                    [ ... ]
                },
                'total': int  # 搜索结果总数
            }
            ```

        - 设置代理：
            ```python
            BaiduSpider().search_baike('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要搜索的关键词.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            dict: 搜索结果和总页数
        """
        error = None
        result = self.EMPTY
        try:
            url = "https://baike.baidu.com/search?word=%s" % quote(query)
            code = self._get_response(url, proxies)
            result = self.parser.parse_baike(code)
            result = result if result is not None else self.EMPTY
        except Exception as err:
            error = err
        finally:
            if error:
                self._handle_error(error)
        return BaikeResult._build_instance(result["results"], result["total"])



"""资讯搜索结果模型类型注释文件

此模块中定义了所有现有资讯搜索内所有模块的结果模型类型注释类，便于现代编辑器自动补全，提供更好的编码体验。
"""
from datetime import datetime
from typing import List, Union


class NewsNormal(object):
    """资讯搜索普通搜索结果模型类型注释类。

    详见`baiduspider.models.news.NewsNormal`类。
    """

    def __init__(self) -> None:
        self.author: str
        self.date: Union[datetime, None]
        self.des: str
        self.title: str
        self.url: str
        self.cover: Union[str, None]
        self.plain: dict


class NewsResult(object):
    """资讯搜索结果模型类型注释类。

    详见`baiduspider.models.news.NewsResult`类。
    """

    def __init__(self) -> None:
        self.results: List[NewsNormal]
        self.pages: int
        self.total: int



"""移动端网页搜索返回值模型模块

此文件定义的所有现有的移动端网页搜索返回值模型并编写了自动构建函数。
"""
from datetime import datetime, time
from baiduspider.mobile.models.typings.typings_web import *
from baiduspider.mobile.models import get_attr, convert_time


class WebVideoDetail(WebVideoDetail):
    """视频详情搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索视频详情搜索结果结果模型创建的返回模型类。

    Attributes:
        author (str): 搜索结果作者（来源）
        author_avatar (str): 搜索结果作者（来源）头像
        duration (datetime.time): 搜索结果时长
        labels (List[str]): 搜索结果标签列表
        poster (str): 搜索结果海报图片链接
        pub_time (datetime.datetime): 搜索结果发表时间
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        video_num (int): 搜索结果“合集”中视频数量
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.author = ""
        self.author_avatar = ""
        self.duration = None
        self.labels = []
        self.poster = ""
        self.pub_time = None
        self.title = ""
        self.url = ""
        self.video_num = 0
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideoDetail:
        __returns = WebVideoDetail()
        __returns.plain = plain
        __returns.author = get_attr(plain, "author")
        __returns.author_avatar = get_attr(plain, "author_avatar")
        try:
            __returns.duration = time(
                minute=int(get_attr(plain, "duration").split(":")[0]),
                second=int(get_attr(plain, "duration").split(":")[1]),
            )
        except:
            __returns.duration = None
        for i in get_attr(plain, "labels"):
            __returns.labels.append(i)
        __returns.poster = get_attr(plain, "poster")
        __returns.pub_time = convert_time(get_attr(plain, "pub_time"))
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.video_num = get_attr(plain, "video_num")
        return __returns

    def __repr__(self) -> str:
        return "<WebVideoDetail %s>" % self.title


class WebVideoTag(WebVideoTag):
    """视频标签搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索视频标签搜索结果结果模型创建的返回模型类。

    Attributes:
        text (str): 标签文字
        url (str): 标签链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.text = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideoTag:
        __returns = WebVideoTag()
        __returns.plain = plain
        __returns.text = get_attr(plain, "text")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebVideoTag %s>" % self.text


class WebVideo(WebVideo):
    """视频搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索视频搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[WebVideoDetail]): 搜索结果详情列表
        tags (List[WebVideoTag]): 搜索结果标签列表
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.results = []
        self.tags = []
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideo:
        __returns = WebVideo()
        __returns.plain = plain
        for i in get_attr(plain, "results"):
            __returns.results.append(WebVideoDetail._build_instance(i))
        for i in get_attr(plain, "tags"):
            __returns.tags.append(WebVideoTag._build_instance(i))
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebVideo %s>" % self.url[:20] + "..."


class WebShortVideoDetail(WebShortVideoDetail):
    """短视频详情搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索短视频详情搜索结果结果模型创建的返回模型类。

    Attributes:
        author (str): 搜索结果作者（来源）
        author_avatar (str): 搜索结果作者（来源）头像
        play_times (int): 搜索结果播放次数
        poster (str): 搜索结果海报图片链接
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.author = ""
        self.author_avatar = ""
        self.play_times = 0
        self.poster = ""
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebShortVideoDetail:
        __returns = WebShortVideoDetail()
        __returns.plain = plain
        __returns.author = get_attr(plain, "author")
        __returns.author_avatar = get_attr(plain, "author_avatar")
        __returns.play_times = get_attr(plain, "play_times")
        __returns.poster = get_attr(plain, "poster")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebShortVideoDetail %s>" % self.title


class WebShortVideo(WebShortVideo):
    """短视频搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索短视频搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[WebShortVideoDetail]): 搜索结果详情列表
        total (int): 搜索结果总数
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.results = []
        self.total = 0
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebShortVideo:
        __returns = WebShortVideo()
        __returns.plain = plain
        if get_attr(plain, "results") is not None:
            for i in get_attr(plain, "results"):
                __returns.results.append(WebShortVideoDetail._build_instance(i))
        __returns.total = get_attr(plain, "total")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebShortVideo %s>" % self.url[:20] + "..."


class WebSection(WebSection):
    """章节搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索章节搜索结果结果模型创建的返回模型类。

    Attributes:
        text (str): 搜索结果文字
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.text = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebSection:
        __returns = WebSection()
        __returns.plain = plain
        __returns.text = get_attr(plain, "text")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebSection %s>" % self.text


class WebBaike(WebBaike):
    """百科搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索百科搜索结果结果模型创建的返回模型类。

    Attributes:
        des (str): 搜索结果简介
        labels (List[str]): 搜索结果标签列表
        origin (str): 搜索结果来源
        poster (str): 搜索结果海报图片链接
        sections (List[WebSection]): 搜索结果章节列表
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.des = ""
        self.labels = []
        self.origin = ""
        self.poster = ""
        self.sections = []
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebBaike:
        __returns = WebBaike()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        for i in get_attr(plain, "labels"):
            __returns.labels.append(i)
        __returns.origin = get_attr(plain, "origin")
        __returns.poster = get_attr(plain, "poster")
        if get_attr(plain, "sections") is not None:
            for i in get_attr(plain, "sections"):
                __returns.sections.append(WebSection._build_instance(i))
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebBaike %s>" % self.title


class WebReyiDetail(WebReyiDetail):
    """热议详情搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索热议详情搜索结果结果模型创建的返回模型类。

    Attributes:
        author (str): 搜索结果作者（来源）
        author_avatar (str): 搜索结果作者（来源）头像
        comments (int): 搜索结果评论数
        des (str): 搜索结果简介
        images (List[str]): 搜索结果图片列表
        likes (int): 搜索结果喜欢数
        origin (str): 搜索结果来源（作者）
        pub_time (datetime.datetime): 搜索结果发布时间
        site (str): 搜索结果发布站点
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.author = ""
        self.author_avatar = ""
        self.comments = 0
        self.des = ""
        self.images = []
        self.likes = 0
        self.origin = ""
        self.pub_time = None
        self.site = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebReyiDetail:
        __returns = WebReyiDetail()
        __returns.plain = plain
        __returns.author = get_attr(plain, "author")
        __returns.author_avatar = get_attr(plain, "author_avatar")
        __returns.comments = get_attr(plain, "comments")
        __returns.des = get_attr(plain, "des")
        if get_attr(plain, "images") is not None:
            for i in get_attr(plain, "images"):
                __returns.images.append(i)
        __returns.likes = get_attr(plain, "likes")
        __returns.origin = get_attr(plain, "origin")
        __returns.pub_time = convert_time(get_attr(plain, "pub_time"))
        __returns.site = get_attr(plain, "site")
        return __returns

    def __repr__(self) -> str:
        return "<WebReyiDetail %s>" % (
            self.des if len(self.des) <= 10 else self.des[:10] + "..."
        )


class WebReyi(WebReyi):
    """热议搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索视频详情搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[WebReyiDetail]): 搜索结果详情列表
        total (int): 搜索结果总数
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.results = []
        self.total = 0
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebReyi:
        __returns = WebReyi()
        __returns.plain = plain
        if get_attr(plain, "results") is not None:
            for i in get_attr(plain, "results"):
                __returns.results.append(WebReyiDetail._build_instance(i))
        __returns.total = get_attr(plain, "total")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebReyi %s>" % self.url[:20] + "..."


class WebKnowledgeDetail(WebKnowledgeDetail):
    """相关知识详情搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索相关知识详情搜索结果结果模型创建的返回模型类。

    Attributes:
        des (str): 搜索结果简介
        image (str): 搜索结果图片链接
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.des = ""
        self.image = ""
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebKnowledgeDetail:
        __returns = WebKnowledgeDetail()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        __returns.image = get_attr(plain, "image")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns

    def __repr__(self) -> str:
        return "<WebKnowledgeDetail %s>" % self.title


class WebKnowledge(WebKnowledge):
    """相关知识搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索相关知识搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[WebKnowledgeDetail]): 搜索结果详情列表
        title (str): 搜索结果标题
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.results = []
        self.title = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebKnowledge:
        __returns = WebKnowledge()
        __returns.plain = plain
        for i in get_attr(plain, "results"):
            __returns.results.append(WebKnowledgeDetail._build_instance(i))
        __returns.title = get_attr(plain, "title")
        return __returns

    def __repr__(self) -> str:
        return "<WebKnowledge %s>" % self.title


class WebNormal(WebNormal):
    """普通搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索普通搜索结果结果模型创建的返回模型类。

    Attributes:
        des (str): 搜索结果简介
        image (str): 搜索结果图片链接
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.des = ""
        self.image = ""
        self.title = ""
        self.url = ""
        self.sections = []
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebNormal:
        __returns = WebNormal()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        __returns.image = get_attr(plain, "image")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        if get_attr(plain, "sections") is not None:
            for i in get_attr(plain, "sections"):
                __returns.sections.append(WebSection._build_instance(i))
        return __returns

    def __repr__(self) -> str:
        return "<WebNormal %s>" % self.title


class WebVideoNormalInfo(WebVideoNormalInfo):
    """普通视频信息搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索普通视频信息搜索结果结果模型创建的返回模型类。

    Attributes:
        data (str): 搜索结果数据
        type (str): 搜索结果类型
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.data = ""
        self.type = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideoNormalInfo:
        __returns = WebVideoNormalInfo()
        __returns.plain = plain
        __returns.data = get_attr(plain, "data")
        __returns.type = get_attr(plain, "type")
        return __returns

    def __repr__(self) -> str:
        return "<WebVideoNormalInfo %s>" % self.type


class WebVideoNormal(WebVideoNormal):
    """普通视频搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索普通视频搜索结果结果模型创建的返回模型类。

    Attributes:
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        poster (str): 搜索结果海报链接
        duration (time): 搜索结果时长
        info (List[WebVideoNormalInfo]): 搜索结果信息列表
        origin (str): 搜索结果来源（作者）
        labels (List[str]): 搜索结果标签列表
        video_num (int): 搜索结果视频数量
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.title = ""
        self.url = ""
        self.poster = ""
        self.duration = None
        self.info = []
        self.origin = ""
        self.labels = []
        self.video_num = 0
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideoNormal:
        __returns = WebVideoNormal()
        __returns.plain = plain
        __returns.poster = get_attr(plain, "poster")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.duration = time(
            minute=int(get_attr(plain, "duration").split(":")[0]),
            second=int(get_attr(plain, "duration").split(":")[1]),
        )
        __returns.origin = get_attr(plain, "origin")
        for i in get_attr(plain, "labels"):
            __returns.info.append(i)
        if get_attr(plain, "info") is not None:
            for i in get_attr(plain, "info"):
                __returns.info.append(WebVideoNormalInfo._build_instance(i))
        __returns.video_num = get_attr(plain, "video_num")
        return __returns

    def __repr__(self) -> str:
        return "<WebVideoNormal %s>" % self.title


class WebResult(WebResult):
    """网页搜索结果模型

    这是一个遵照BaiduSpider移动端网页搜索结果结果模型创建的返回模型类。

    Attributes:
        video (Union[WebVideo, None]): 视频搜索结果
        short_video (Union[WebShortVideo, None]): 短视频搜索结果
        baike (Union[WebBaike, None]): 百科搜索结果
        reyi (Union[WebReyi, None]): 热议搜索结果
        knowledge (Union[WebKnowledge, None]): 相关知识搜索结果
        normal (List[WebNormal]): 普通搜索结果列表
        video_normal (List[WebVideoNormal]): 普通视频搜索结果列表
        query (str): 搜索词
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.video = None
        self.short_video = None
        self.baike = None
        self.reyi = None
        self.knowledge = None
        self.normal = []
        self.video_normal = []
        self.query = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: list, query: str = "") -> WebResult:
        __returns = WebResult()
        __returns.plain = plain
        __returns.query = query
        for p in plain:
            if get_attr(p, "type") == "result":
                __returns.normal.append(WebNormal._build_instance(p))
            elif get_attr(p, "type") == "video":
                __returns.video = WebVideo._build_instance(p)
            elif get_attr(p, "type") == "short_video":
                __returns.short_video = WebShortVideo._build_instance(p)
            elif get_attr(p, "type") == "reyi":
                __returns.reyi = WebReyi._build_instance(p)
            elif get_attr(p, "type") == "knowledge":
                __returns.knowledge = WebKnowledge._build_instance(p)
            elif get_attr(p, "type") == "baike":
                __returns.baike = WebBaike._build_instance(get_attr(p, "result"))
            elif get_attr(p, "type") == "video_normal":
                __returns.video_normal.append(WebVideoNormal._build_instance(p))
        return __returns

    def __repr__(self) -> str:
        return "<WebResult %s>" % self.query



"""视频搜索返回值模型模块

此文件定义的所有现有的视频搜索返回值模型并编写了自动构建函数。
"""
from typing import Union

from baiduspider.models import get_attr
from baiduspider.models.typings.typings_video import *


class VideoNormal(VideoNormal):
    """视频搜索普通搜索结果模型

    这是一个遵照BaiduSpider视频搜索基本搜索结果结果模型创建的返回模型类。

    Attributes:
        des (str): 视频简介
        img (str): 视频封面图片链接
        origin (str | None): 视频来源（作者）
        pub_time (datetime.datetime): 视频发布时间
        length (datetime.time): 视频时长
        title (str): 视频标题
        url (str): 视频链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.img = ""
        self.length = None
        self.pub_time = None
        self.des = ""
        self.title = ""
        self.url = ""
        self.origin = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> VideoNormal:
        __returns = VideoNormal()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        __returns.img = get_attr(plain, "img")
        __returns.origin = get_attr(plain, "origin")
        __returns.pub_time = get_attr(plain, "pub_time")
        __returns.length = get_attr(plain, "length")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns


class VideoResult(VideoResult):
    """视频搜索结果模型

    这是一个遵照BaiduSpider视频搜索结果结果模型创建的返回模型类。

    Attributes:
        results (List[VideoNormal]): 普通搜索结果列表
        is_last (bool): 当前页数是否为最后一页
        plain (list): 源搜索结果列表
    """

    def __init__(self) -> None:
        super().__init__()
        self.results = []
        self.is_last = False
        self.plain = []

    @staticmethod
    def _build_instance(plain: list) -> VideoResult:
        __returns = VideoResult()
        __returns.plain = plain
        if plain is None:
            __returns.is_last = True
            return __returns
        for p in plain:
            __returns.results.append(VideoNormal._build_instance(p))
        return __returns

    def __getitem__(self, key) -> Union[VideoNormal, None]:
        return self.results[key]

    def __repr__(self) -> str:
        return "<object VideoResult>"



import re
from datetime import datetime, timedelta
from typing import Union
from baiduspider.util import convert_time


def get_attr(d: dict, t: str):
    """获取字典`d`下的`t`"""
    try:
        return d[t]
    except:
        return None



import inspect
import os
from functools import wraps
from datetime import datetime, timedelta
import re
from typing import Union


def handle_err(func):  # pragma: no cover
    """处理报错"""

    @wraps(func)
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except Exception as err:
            if bool(int(os.environ.get("DEBUG", 0))):
                raise err
            func_name = (
                func.__name__.strip("parse_").strip("_block")
                if "block" in func.__name__
                else None
            )
            if func_name is not None:
                print(
                    f"\033[33mWARNING: An error occurred while parsing the {func_name} subcomponent of BaiduSpider.{inspect.stack()[1][0].f_code.co_name}, "
                    "which is currently ignored. However, the rest of the parsing process is still being executed normally. "
                    "This is most likely an inner parse failure of BaiduSpider. For more details, please set the environment "
                    "variable `DEBUG` to `1` to see the error trace and open up a new issue at https://github.com/BaiduSpider/"
                    "BaiduSpider/issues/new?assignees=&labels=bug%2C+help+wanted&template=bug_report.md&title=%5BBUG%5D.\033[0m"
                )
            else:
                print(
                    f"\033[33mWARNING: An error occurred while executing function BaiduSpider.{inspect.stack()[1][0].f_code.co_name}, "
                    "which is currently ignored. However, the rest of the parsing process is still being executed normally. "
                    "This is most likely an inner parse failure of BaiduSpider. For more details, please set the environment "
                    "variable `DEBUG` to `1` to see the error trace and open up a new issue at https://github.com/BaiduSpider/"
                    "BaiduSpider/issues/new?assignees=&labels=bug%2C+help+wanted&template=bug_report.md&title=%5BBUG%5D.\033[0m"
                )

    return wrapper


def convert_time(t: str, as_list: bool = False) -> Union[datetime, bool]:
    """转换有时间差的汉字表示的时间到`datetime.datetime`形式的时间

    Args:
        t (str): 要转换的字符串
        as_list (bool): 是否以列表形式返回

    Returns:
        datetime: 转换后的`datetime.datetime`结果
    """
    if t is None:
        return None
    t = t.strip()
    mappings = {"昨天": 1, "前天": 2, "今天": 0}
    if t in mappings:
        return datetime.now() - timedelta(days=mappings[t])
    delta = int(re.findall("\d+", t)[0])
    # print( t.replace(str(delta), "").strip(), delta)
    if "秒" in t:
        s = datetime.now() - timedelta(seconds=delta)
    elif "分钟" in t:
        s = datetime.now() - timedelta(minutes=delta)

    elif "小时" in t:
        s = datetime.now() - timedelta(hours=delta)
    elif t.replace(str(delta), "").split(":")[0].strip() in mappings:
        _ = int(re.findall("\d+", t)[-1])
        __ = t.replace(str(delta), "").split(":")[0].strip()
        s = datetime.now() - timedelta(days=mappings[__])
        s = datetime(s.year, s.month, s.day, delta, _)
    elif "天" in t:
        s = datetime.now() - timedelta(days=delta)
    # elif '年' in t:
    #     s = (datetime.now() - timedelta(days=365 * delta))
    elif "年" in t and "月" in t and "日" in t:
        s = datetime.strptime(t, "%Y年%m月%d日")
    else:
        s = datetime.now()
    if not as_list:
        return s
    return [s.year, s.month, s.day, s.hour, s.minute, s.second]



"""BaiduSpider移动端爬虫"""
import datetime
import time as time_lib
from time import mktime, strptime, time
from typing import Tuple, Union
from urllib.parse import SplitResult, quote

import requests
from bs4 import BeautifulSoup

from baiduspider._spider import BaseSpider
from baiduspider.mobile.parser import MobileParser
from baiduspider.mobile.models.web import WebResult


__all__ = ["BaiduMobileSpider"]


class BaiduMobileSpider(BaseSpider):
    def __init__(self) -> None:
        """爬取百度移动端的搜索结果.

        本类的所有成员方法都遵循下列格式：

            {
                'results': <一个列表，表示搜索结果，内部的字典会因为不同的成员方法而改变>,
                'total': <一个正整数，表示搜索结果的最大页数，可能会因为搜索结果页码的变化而变化，因为百度不提供总共的搜索结果页数>
            }

        目前仅支持百度网页搜索的部分内容爬取，返回的搜索结果无广告。继承自``BaseSpider``。

        BaiduMobileSpider.`search_web(self: BaiduMobileSpider, query: str, pn: int = 1) -> WebResult`: 移动端百度网页搜索
        """
        super().__init__()
        # 爬虫名称（不是请求的，只是用来表识）
        self.spider_name = "BaiduSpider"
        # 设置请求头
        self.headers = {
            "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/91.0.4472.114 Safari/537.36 Edg/91.0.864.59",
            "Referer": "https://m.baidu.com",
            "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",
            "Accept-Encoding": "gzip, deflate, br",
            "Accept-Language": "en-US,en;q=0.9,zh-CN;q=0.8,zh;q=0.7",
        }
        self.parser = MobileParser()
        self.EMPTY = {"results": []}

    def search_web(self, query: str, pn: int = 1, proxies: dict = None) -> WebResult:
        """搜索百度移动端网页搜索结果。

        - 基本使用：
            ```python
            BaiduMobileSpider().search_web('搜索词')
            ```

        - 带页码：
            ```python
            BaiduMobileSpider().search_web('搜索词', pn=2)
            ```

        - 设置代理：
            ```python
            BaiduMobileSpider().search_web('搜索词', proxies={
                "http": "http://xxx.xxx.xxx:xxxx",  # HTTP代理
                "https": "https://xxx.xxx.xxx:xxxx"  # HTTPS代理
            })
            ```
            详细配置请参考[requests文档](https://docs.python-requests.org/zh_CN/latest/user/advanced.html?highlight=proxies#proxies)。

        Args:
            query (str): 要爬取的搜索词.
            pn (int, optional): 爬取的页码. Defaults to 1.
            proxies (Union[dict, None]): 代理配置. Defaults to None.

        Returns:
            WebResult: 爬取的返回值和搜索结果
        """
        error = None
        try:
            text = quote(query, "utf-8")
            url = "https://m.baidu.com/s?word=%s&pn=%d" % (text, ((pn - 1) * 10))
            content = self._get_response(url, proxies)
            results = self.parser.parse_web(content)
        except Exception as err:
            error = err
        finally:
            self._handle_error(error, "BaiduSpider", "parse-web")
        return WebResult._build_instance(results["results"], query)



import json
from datetime import time, datetime

import requests
from baiduspider._spider import BaseSpider
from baiduspider.util import handle_err
from bs4 import BeautifulSoup, Comment


class WebSubParser(BaseSpider):
    """网页搜索子解析模块。

    此模块为`BaiduSpider.search_web`函数的子模块，用于解析网页搜索子模块的HTML代码
    并返回Python字典。
    """

    def __init__(self) -> None:
        super().__init__()
        self.spider_name = "WebSubSpider"

    @handle_err
    def parse_news_block(self, news: BeautifulSoup) -> dict:
        """解析资讯子块

        Args:
            news (BeautifulSoup): 从源HTML代码中提取的资讯块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        try:
            self._format(news.find("h3", class_="t").find("a").text)
        except:
            news_detail = []
        else:
            news_rows = news.findAll("div", class_="c-row")
            news_detail = []
            prev_row = {}
            for row in news_rows:
                try:
                    row_title = self._format(row.find("a").text)
                except AttributeError:
                    prev_row["des"] = self._format(row.text)
                    continue
                row_time = self._format(row.find("span", class_="c-color-gray2").text)
                row_author = self._format(row.find("span", class_="c-color-gray").text)
                row_url = self._format(row.find("a")["href"])
                news_detail.append(
                    {
                        "title": row_title,
                        "time": row_time,
                        "author": row_author,
                        "url": row_url,
                        "des": None,
                    }
                )
                prev_row = news_detail[-1]
        return news_detail

    @handle_err
    def parse_video_block(self, video: BeautifulSoup) -> dict:
        """解析视频子块

        Args:
            video (BeautifulSoup): 从源HTML代码中提取的视频块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if video:
            video_rows = video.findAll("div", class_="c-row")
            video_results = []
            for row in video_rows:
                row_res = []
                videos = row.findAll("div", class_="c-span3")
                for v in videos:
                    v_link = v.find("a")
                    v_title = v_link["title"]
                    v_url = self._format(v_link["href"])
                    v_img = v_link.find("img")["src"]
                    v_len = self._format(
                        v.find("div", class_="op-short-video-pc-duration-wrap-new").text
                    )
                    v_from = self._format(
                        v.find("div", class_="op-short-video-pc-clamp1").text
                    )
                    row_res.append(
                        {
                            "title": v_title,
                            "url": v_url,
                            "cover": v_img,
                            "length": v_len,
                            "origin": v_from,
                        }
                    )
                video_results += row_res
        else:
            video_results = []
        return video_results

    @handle_err
    def parse_baike_block(self, baike: BeautifulSoup) -> dict:
        """解析百科子块

        Args:
            baike (BeautifulSoup): 从源HTML代码中提取的百科块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if baike:
            b_title = self._format(baike.find("h3").text)
            b_url = baike.find("a")["href"]
            b_des = self._format(
                baike.find("div", class_="c-span-last")
                .find("div", class_="c-font-normal")
                .text
            )
            try:
                b_cover = baike.find("div", class_="c-span3").find("img")["src"]
                b_cover_type = "image"
            except (TypeError, AttributeError):
                try:
                    b_cover = (
                        baike.find("div", class_="op-bk-polysemy-imgWrap")
                        .find("div", class_="c-img")["style"]
                        .split("url", 1)[-1]
                        .split(")", 1)[0]
                        .strip("(")
                    )
                    b_cover_type = "video"
                except (TypeError):
                    b_cover = None
                    b_cover_type = None
            baike = {
                "title": b_title,
                "url": b_url,
                "des": b_des,
                "cover": b_cover,
                "cover-type": b_cover_type,
            }
        return baike

    @handle_err
    def parse_tieba_block(self, tieba: BeautifulSoup) -> dict:
        """解析贴吧子块

        Args:
            tieba (BeautifulSoup): 从源HTML代码中提取的贴吧块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if tieba:
            t_title = self._format(tieba.find("h3").text)
            t_url = tieba["mu"]
            try:
                t_info_ = tieba.find(
                    "div", class_="op-tieba-general-col-top-xs"
                ).findAll("p")
                t_des = self._format(t_info_[0].text)
            except AttributeError:
                t_des = None
            t_followers = self._format(
                tieba.find("div", class_="c-span-last").find("span").find("span").text
            )
            t_total = self._format(
                tieba.find("div", class_="c-span-last").findAll("span")[-1].text
            )
            try:
                t_cover = tieba.find("a", class_="op-tieba-general-photo-link").find(
                    "img"
                )["src"]
            except AttributeError:
                t_cover = None
            t_hot_ = tieba.findAll("div", class_="c-row")[1:-1]
            t_hot = []
            i = 1
            for hot in t_hot_:
                t_h_title = self._format(hot.find("a").text)
                t_h_url = hot.find("a")["href"]
                t_h_clicks = self._format(
                    hot.find("div", class_="c-color-gray2").find("span").text
                )
                t_h_replies = self._format(
                    hot.findAll("div", class_="c-color-gray2")[-1].find("span").text
                )
                t_hot.append(
                    {
                        "title": t_h_title,
                        "url": t_h_url,
                        "clicks": t_h_clicks,
                        "replies": t_h_replies,
                    }
                )
                i += 1
            del i
            tieba = {
                "title": t_title,
                "url": t_url,
                "des": t_des,
                "followers": t_followers,
                "total": t_total,
                "cover": t_cover,
                "hot": t_hot,
            }
        return tieba

    @handle_err
    def parse_blog_block(self, blog: BeautifulSoup) -> dict:
        """解析博客子块

        Args:
            blog (BeautifulSoup): 从源HTML代码中提取的博客块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if blog is not None:
            blog = blog.find("section")
            b_title = blog.find("h3", class_="c-title").text
            b_url = blog.find("a")["href"]
            b_blogs_ = blog.findAll("div", class_="c-row")
            b_blogs = []
            for b in b_blogs_:
                b_current_blog_header = b.find("div")
                b_blog_title = b_current_blog_header.find("a").text
                b_blog_url = b_current_blog_header.find("a")["href"]
                b_blog_origin = b_current_blog_header.find(
                    "span", class_="nor-src-wrap"
                ).text
                try:
                    b_current_blog_tags = b.findAll("div")[1].findAll("span")
                    b_blog_tags = [tag.text for tag in b_current_blog_tags]
                except IndexError:
                    b_blog_tags = []
                b_blog_parent = b.find_parent("div").findAll("div")
                b_blog_des = None
                for p in b_blog_parent:
                    if p["class"][0].startswith("blog-summary"):
                        b_blog_des = p.text
                        break
                b_blogs.append(
                    {
                        "title": b_blog_title,
                        "url": b_blog_url,
                        "origin": b_blog_origin,
                        "tags": b_blog_tags,
                        "des": b_blog_des,
                    }
                )
            blog = {"title": b_title, "url": b_url, "blogs": b_blogs}
        return blog

    @handle_err
    def parse_gitee_block(self, gitee: BeautifulSoup) -> dict:
        """解析Gitee仓库子块

        Args:
            gitee (BeautifulSoup): 从源HTML代码中提取的码云仓库块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if gitee is not None:
            g_title = gitee.find("h3", class_="c-title").text
            g_url = gitee.find("a", class_="c-blocka")["href"]
            gitee = gitee.find("section").find("div", class_="c-tabs-content-wrapper")
            g_tabs = gitee.findAll("div", class_="c-tabs-content")
            g_intro = g_tabs[0].find("div", class_="c-tabs-item").find("div")
            g_des = g_intro.find("div").text
            g_license = (
                g_intro.findAll("div")[1].text.lstrip("开源协议：")
                if len(g_intro.findAll("div")) >= 2
                and g_intro.findAll("div")[1].text.startswith("开源协议：")
                else None
            )
            g_lang = (
                g_intro.findAll("div")[2].text.lstrip("开发语言：")
                if len(g_intro.findAll("div")) >= 2
                and g_intro.findAll("div")[1].text.startswith("开发语言：")
                or len(g_intro.findAll("div")) >= 3
                and g_intro.findAll("div")[2].text.startswith("开发语言：")
                else None
            )
            g_temp = g_intro.findAll("span")
            g_star = int(g_temp[0].text.strip("Star："))
            g_fork = int(g_temp[1].text.strip("Fork："))
            g_watch = int(g_temp[2].text.strip("Watch："))
            g_status = g_tabs[-1].find("img")["src"]
            gitee = {
                "title": g_title,
                "url": g_url,
                "des": g_des,
                "license": g_license,
                "lang": g_lang,
                "star": g_star,
                "fork": g_fork,
                "watch": g_watch,
                "status": g_status,
            }
        return gitee

    @handle_err
    def parse_music_block(self, music: BeautifulSoup) -> dict:
        """解析音乐子块

        Args:
            music (BeautifulSoup): 从源HTML代码中提取的音乐块BeautifulSoup对象

        Returns:
            dict: 解析后自动生成的Python结果字典对象
        """
        if music is not None:
            # 从注释中获取结果JSON
            music = json.loads(
                music.find(text=lambda text: isinstance(text, Comment)).strip(
                    "s-data: "
                )
            )
            m_title = music["title"].replace("<em>", "").replace("</em>", "")  # 搜索结果标题
            m_url = music["url"]  # 搜索结果链接
            m_songs = []  # 搜索结果歌曲
            for song in music["data"]["site"]:
                # 歌手信息
                s_singer = [
                    {"url": i["singerUrl"], "name": i["singerName"]}
                    for i in song["singer"]
                ]
                # 预处理歌曲发布时间
                try:
                    __ = song["publishTime"].split("-")
                except KeyError:
                    __ = None
                # 预处理歌曲时长
                _ = int(song["duration"])
                # 歌曲信息
                s_song = {
                    "name": song["displaySongName"],  # 歌曲名称
                    "url": song["songUrl"],  # 歌曲链接
                    "poster": song["poster"],  # 歌曲海报图片链接
                    "is_original": bool(int(song["isOriginal"])),  # 是否为原唱
                    "pub_date": datetime(int(__[0]), int(__[1]), int(__[2]))
                    if __ is not None
                    else None,  # 歌曲发布时间
                    "labels": [i["txt"] for i in song["labels"]],  # 歌曲标签
                    "copyright": bool(int(song["copyRight"])),  # 歌曲是否有版权
                    "site": song["sitePinyin"],  # 歌曲发布站点（拼音）
                    "duration": time(
                        hour=int(_ / 60 / 60), minute=int(_ / 60), second=int(_ % 60)
                    ),  # 歌曲时长
                    "other_sites": song["allWapPlayFile"],  # 歌曲其他网站链接
                }
                # 歌曲发布公司
                try:
                    s_song["pub_company"] = song["pubCompany"]
                    if song["pubCompany"] == "null":
                        s_song["pub_company"] = None
                except KeyError:
                    s_song["pub_company"] = None
                # 歌曲专辑
                try:
                    s_album = {
                        "url": song["album"]["albumUrl"],
                        "name": song["album"]["albumName"],
                    }
                except KeyError:
                    s_album = None
                m_songs.append({"song": s_song, "singer": s_singer, "album": s_album})
            music = {"title": m_title, "url": m_url, "songs": m_songs}
        return music



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="BaiduSpider",
    version="1.0.1.3",
    author="Sam Zhang",
    author_email="samzhang951@outlook.com",
    description="BaiduSpider，一个爬取百度的利器",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/BaiduSpider/BaiduSpider",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3 :: Only",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "License :: Free for non-commercial use",
        "Topic :: Software Development :: Libraries",
        "Topic :: Utilities",
    ],
    python_requires=">=3.6",
    install_requires=["requests", "bs4"],
)



"""搜索词预测模型类型注释文件

此文件定义了所有现有的搜索词预测模块内返回值模型并编写了自动构建函数。
"""
from typing import Union
from baiduspider.predictor.models.typings import TiebaPredictorResult


def get_attr(d: dict, t: str):
    """获取字典`d`下的`t`"""
    try:
        return d[t]
    except:
        return None


class TiebaPredictorResult(TiebaPredictorResult):
    """贴吧搜索搜索结果预测结果模型类型注释类。

    这是一个遵照BaiduSpider百科搜索结果结果预测模型创建的返回模型类。

    Attributes:
        classifiers (List[str]): 预测结果类别分类
        cover (str): 预测结果封面图片URL
        desc (str): 预测结果介绍
        members (int): 预测结果成员数
        name (str): 预测结果名称
        threads (int): 预测结果帖子数
        plain (list): 源预测结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.classifiers = []
        self.cover = ""
        self.desc = ""
        self.members = 0
        self.name = ""
        self.threads = 0
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> TiebaPredictorResult:
        __returns = TiebaPredictorResult()
        __returns.plain = plain
        __returns.classifiers = get_attr(plain, "classifiers")
        __returns.cover = get_attr(plain, "cover")
        __returns.desc = get_attr(plain, "desc")
        __returns.members = get_attr(plain, "members")
        __returns.name = get_attr(plain, "name")
        __returns.threads = get_attr(plain, "threads")
        return __returns

    def __repr__(self) -> str:
        return "<object TiebaPredictorResult>"



"""网页搜索返回值模型模块

此文件定义的所有现有的网页搜索返回值模型并编写了自动构建函数。
"""
from typing import Any

from baiduspider.models import convert_time, get_attr
from baiduspider.models.typings.typings_web import *


class WebNormal(WebNormal):
    """普通搜索结果模型

    这是一个遵照BaiduSpider网页搜索基本搜索结果结果模型创建的返回模型类。

    Attributes:
        des (str): 搜索结果简介
        origin (str): 搜索结果来源
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        time (datetime): 搜索结果发布时间
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.des = ""
        self.origin = ""
        self.title = ""
        self.url = ""
        self.time = None
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebNormal:
        __returns = WebNormal()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        __returns.origin = get_attr(plain, "origin")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.time = convert_time(get_attr(plain, "time"))
        return __returns


class WebCalc(WebCalc):
    """计算搜索结果模型

    这是一个遵照BaiduSpider网页搜索计算搜索结果结果模型创建的返回模型类。

    Attributes:
        process (str): 计算过程，如：12 + 21
        result (str): 计算结果，由于可能是`1e6`这样的形式，所以类型为`str`
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.process = ""
        self.result = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebCalc:
        __returns = WebCalc()
        __returns.plain = plain
        __returns.process = get_attr(plain, "process")
        __returns.result = int(get_attr(plain, "result"))
        return __returns


class WebNews(WebNews):
    """资讯搜索结果模型

    这是一个遵照BaiduSpider网页搜索资讯搜索结果结果模型创建的返回模型类。

    Attributes:
        author (str): 资讯作者（来源）
        time (datetime): 资讯发布时间
        url (str): 资讯链接
        des (str): 资讯简介
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.author = ""
        self.time = None
        self.url = ""
        self.des = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebNews:
        __returns = WebNews()
        __returns.plain = plain
        __returns.author = get_attr(plain, "author")
        __returns.time = convert_time(get_attr(plain, "time"))
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.des = get_attr(plain, "des")
        return __returns


class WebVideo(WebVideo):
    """视频搜索结果模型

    这是一个遵照BaiduSpider网页搜索视频搜索结果结果模型创建的返回模型类。

    Attributes:
        cover (str): 视频封面图片链接
        origin (str): 视频来源
        length (str): 视频时长
        title (str): 视频标题
        url (str): 视频链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.cover = ""
        self.origin = ""
        self.length = ""
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebVideo:
        __returns = WebVideo()
        __returns.plain = plain
        __returns.cover = get_attr(plain, "cover")
        __returns.origin = get_attr(plain, "origin")
        __returns.length = get_attr(plain, "length")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebBaike(WebBaike):
    """百科搜索结果模型

    这是一个遵照BaiduSpider网页搜索百科搜索结果结果模型创建的返回模型类。

    Attributes:
        cover (str): 百科封面图片（视频）链接
        cover_type (str): 百科封面类型，图片时为"image"，视频时为"video"
        des (str): 百科简介
        title (str): 百科标题
        url (str): 百科链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.cover = ""
        self.cover_type = ""
        self.des = ""
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebBaike:
        __returns = WebBaike()
        __returns.plain = plain
        __returns.cover = get_attr(plain, "cover")
        __returns.cover_type = get_attr(plain, "cover-type")
        __returns.des = get_attr(plain, "des")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebTiebaHot(WebTiebaHot):
    """贴吧热门搜索结果模型

    这是一个遵照BaiduSpider网页搜索贴吧热门搜索结果结果模型创建的返回模型类。

    Attributes:
        clicks (str): 帖子点击次数，可能会有形如`1万`的结果，故类型为`str`
        replies (str): 帖子回复次数，可能会有形如`1万`的结果，故类型为`str`
        title (str): 帖子标题
        url (str): 帖子链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.clicks = ""
        self.replies = ""
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebTiebaHot:
        __returns = WebTiebaHot()
        __returns.plain = plain
        __returns.clicks = get_attr(plain, "clicks")
        __returns.replies = get_attr(plain, "replies")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebTieba(WebTieba):
    """贴吧搜索结果模型

    这是一个遵照BaiduSpider网页搜索贴吧搜索结果结果模型创建的返回模型类。

    Attributes:
        cover (str): 贴吧封面图片链接
        des (str): 贴吧简介
        title (str): 贴吧标题
        followers (str): 贴吧关注人数，可能会有形如`1万`的结果，故类型为`str`
        total (str): 贴吧总帖子数，可能会有形如`1万`的结果，故类型为`str`
        hot (List[WebTiebaHot]): 贴吧热门帖子
        url (str): 贴吧链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.cover = ""
        self.des = ""
        self.title = ""
        self.followers = ""
        self.total = ""
        self.url = ""
        self.hot = []
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebTieba:
        __returns = WebTieba()
        __returns.plain = plain
        __returns.cover = get_attr(plain, "cover")
        __returns.des = get_attr(plain, "des")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.followers = get_attr(plain, "followers")
        __returns.total = get_attr(plain, "total")
        __returns.url = get_attr(plain, "url")
        for i in get_attr(plain, "hot"):
            __returns.hot.append(WebTiebaHot._build_instance(i))
        return __returns


class WebBlogDetail(WebBlogDetail):
    """博客详情搜索结果模型

    这是一个遵照BaiduSpider网页搜索博客详情搜索结果结果模型创建的返回模型类。

    Attributes:
        title (str): 博客标题
        url (str): 博客链接
        des (str): 博客简介
        origin (str): 博客来源（作者）
        tags (List[str]): 博客标签
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.title = ""
        self.url = ""
        self.des = ""
        self.origin = ""
        self.tags = []
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebBlogDetail:
        __returns = WebBlogDetail()
        __returns.plain = plain
        __returns.des = get_attr(plain, "des")
        __returns.origin = get_attr(plain, "origin")
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        for i in get_attr(plain, "tags"):
            __returns.tags.append(i)
        return __returns


class WebBlog(WebBlog):
    """博客搜索结果模型

    这是一个遵照BaiduSpider网页搜索博客搜索结果结果模型创建的返回模型类。

    Attributes:
        title (str): 博客搜索结果标题
        url (str): 博客搜索结果链接
        blogs (List[WebBlogDetail]): 博客详情列表
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.title = ""
        self.url = ""
        self.blogs = []
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebBlog:
        __returns = WebBlog()
        __returns.plain = plain
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        for i in get_attr(plain, "blogs"):
            __returns.blogs.append(WebBlogDetail._build_instance(i))
        return __returns


class WebGitee(WebGitee):
    """Gitee仓库搜索结果模型

    这是一个遵照BaiduSpider网页搜索Gitee仓库搜索结果结果模型创建的返回模型类。

    Attributes:
        title (str): 仓库标题
        des (str): 仓库简介
        url (str): 仓库链接
        star (int): 仓库star数
        fork (int): 仓库fork数
        watch (int): 仓库watch数
        license (str): 仓库使用的开源协议
        lang (str): 仓库使用的编程语言
        status (str): 仓库状态（码云指数）图片链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        super().__init__()
        self.title = ""
        self.des = ""
        self.url = ""
        self.star = 0
        self.fork = 0
        self.watch = 0
        self.license = ""
        self.lang = ""
        self.status = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebGitee:
        __returns = WebGitee()
        __returns.plain = plain
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        __returns.des = get_attr(plain, "des")
        __returns.star = get_attr(plain, "star")
        __returns.fork = get_attr(plain, "fork")
        __returns.watch = get_attr(plain, "watch")
        __returns.license = get_attr(plain, "license")
        __returns.lang = get_attr(plain, "lang")
        __returns.status = get_attr(plain, "status")
        return __returns


class WebMusicSong(WebMusicSong):
    """网页搜索音乐歌曲信息搜索结果结果模型

    这是一个遵照BaiduSpider网页搜索音乐歌曲信息搜索结果结果模型创建的返回模型类。

    Attributes:
        copyright (bool): 歌曲是否有版权限制
        duration (datetime.time): 歌曲时长
        is_original (bool): 歌曲是否为原唱
        labels (List[str]): 歌曲标签
        name (str): 歌曲名称
        other_sites (List[str]): 歌曲其他网站链接
        poster (str): 歌曲海报图片链接
        pub_company (str | None): 歌曲发布公司
        pub_date (datetime.datetime | None): 歌曲发布日期
        site (str): 歌曲发布站点（拼音）
        url (str): 歌曲链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.copyright = False
        self.duration = None
        self.is_original = False
        self.labels = []
        self.name = ""
        self.other_sites = []
        self.poster = ""
        self.pub_company = None
        self.pub_date = None
        self.site = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebMusicSong:
        __returns = WebMusicSong()
        __returns.plain = plain
        __returns.copyright = get_attr(plain, "copyright")
        __returns.duration = get_attr(plain, "duration")
        __returns.is_original = get_attr(plain, "is_original")
        __returns.labels = get_attr(plain, "labels")
        __returns.name = get_attr(plain, "name")
        __returns.other_sites = get_attr(plain, "other_sites")
        __returns.poster = get_attr(plain, "poster")
        __returns.pub_company = get_attr(plain, "pub_company")
        __returns.pub_date = get_attr(plain, "pub_date")
        __returns.site = get_attr(plain, "site")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebMusicAlbum(WebMusicAlbum):
    """网页搜索音乐歌曲专辑搜索结果结果模型。

    这是一个遵照BaiduSpider网页搜索音乐歌曲专辑搜索结果结果模型创建的返回模型类。

    Attributes:
        name (str): 专辑名称
        url (str): 专辑链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.name = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebMusicAlbum:
        __returns = WebMusicAlbum()
        __returns.plain = plain
        __returns.name = get_attr(plain, "name")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebMusicSinger(WebMusicSinger):
    """网页搜索音乐歌曲歌手搜索结果结果模型。

    这是一个遵照BaiduSpider网页搜索音乐歌曲歌手搜索结果结果模型创建的返回模型类。

    Attributes:
        name (str): 歌手名称
        url (str): 歌手链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.name = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebMusicSinger:
        __returns = WebMusicSinger()
        __returns.plain = plain
        __returns.name = get_attr(plain, "name")
        __returns.url = get_attr(plain, "url")
        return __returns


class WebMusicSongs(WebMusicSongs):
    """网页搜索音乐歌曲搜索结果结果模型。

    这是一个遵照BaiduSpider网页搜索音乐歌曲搜索结果结果模型创建的返回模型类。

    Attributes:
        album (WebMusicAlbum): 歌曲专辑
        singers (List[WebMusicSinger]): 歌曲歌手
        song (WebMusicSong): 歌曲信息
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.album = None
        self.singers = []
        self.song = None
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebMusicSongs:
        __returns = WebMusicSongs()
        __returns.plain = plain
        __returns.album = WebMusicAlbum._build_instance(get_attr(plain, "album"))
        for i in get_attr(plain, "singer"):
            __returns.singers.append(WebMusicSinger._build_instance(i))
        __returns.song = WebMusicSong._build_instance(get_attr(plain, "song"))
        return __returns


class WebMusic(WebMusic):
    """网页搜索音乐搜索结果结果模型。

    这是一个遵照BaiduSpider网页搜索音乐搜索结果结果模型创建的返回模型类。

    Attributes:
        songs (List[WebMusicSongs]): 歌曲信息
        title (str): 搜索结果标题
        url (str): 搜索结果链接
        plain (dict): 源搜索结果字典
    """

    def __init__(self) -> None:
        self.songs = []
        self.title = ""
        self.url = ""
        self.plain = {}

    @staticmethod
    def _build_instance(plain: dict) -> WebMusic:
        __returns = WebMusic()
        __returns.plain = plain
        __returns.title = get_attr(plain, "title")
        __returns.url = get_attr(plain, "url")
        for i in get_attr(plain, "songs"):
            __returns.songs.append(WebMusicSongs._build_instance(i))
        return __returns


class WebResult(WebResult):
    """网页搜索结果模型

    这是一个遵照BaiduSpider网页搜索结果结果模型创建的返回模型类。

    Attributes:
        normal (List[WebNormal]): 普通搜索结果列表
        total (int): 搜索结果总数
        related (List[str]): 相关搜索结果列表
        calc (WebCalc | None): 计算搜索结果
        news (List[WebNews]): 资讯搜索结果列表
        video (List[WebVideo]): 视频搜索结果列表
        baike (WebBaike | None): 百科搜索结果
        tieba (WebTieba | None): 贴吧搜索结果
        blog (WebBlog | None): 博客搜索结果
        gitee (WebGitee | None): 码云仓库搜索结果
        music (WebMusic | None): 音乐搜索结果
        pages (int): 搜索结果页数
        total (int): 搜索结果总数
        plain (list): 源搜索结果列表
    """

    def __init__(self) -> None:
        super().__init__()
        self.normal = []
        self.total = 0
        self.related = []
        self.calc = None
        self.news = []
        self.video = []
        self.baike = None
        self.tieba = None
        self.blog = None
        self.gitee = None
        self.music = None
        self.pages = 0
        self.plain = []
        self.total = 0

    @staticmethod
    def _build_instance(plain: list, pages: int, total: int) -> WebResult:
        __returns = WebResult()
        __returns.plain = plain
        __returns.pages = pages
        __returns.total = total
        for p in plain:
            if get_attr(p, "type") == "result":
                __returns.normal.append(WebNormal._build_instance(p))
            elif get_attr(p, "type") == "total":
                __returns.total = get_attr(p, "result")
            elif get_attr(p, "type") == "related":
                __returns.related = get_attr(p, "results")
            elif get_attr(p, "type") == "calc":
                __returns.calc = WebCalc._build_instance(p)
            elif get_attr(p, "type") == "news":
                __returns.news = [
                    WebNews._build_instance(i) for i in get_attr(p, "results")
                ]
            elif get_attr(p, "type") == "video":
                __returns.video = [
                    WebVideo._build_instance(i) for i in get_attr(p, "results")
                ]
            elif get_attr(p, "type") == "baike":
                __returns.baike = WebBaike._build_instance(get_attr(p, "result"))
            elif get_attr(p, "type") == "tieba":
                __returns.baike = WebTieba._build_instance(get_attr(p, "result"))
            elif get_attr(p, "type") == "blog":
                __returns.blog = WebBlog._build_instance(get_attr(p, "result"))
            elif get_attr(p, "type") == "gitee":
                __returns.gitee = WebGitee._build_instance(get_attr(p, "result"))
            elif get_attr(p, "type") == "music":
                __returns.music = WebMusic._build_instance(get_attr(p, "result"))
        return __returns

    def __getitem__(self, key) -> Any:
        return self.plain[key]

    def __repr__(self) -> str:
        return "<object WebResult>"
